#!/usr/bin/python
#
# Copyright (c) 2021 Ross Bender (@l3ender)
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: azure_rm_monitordiagnosticsetting
version_added: "1.10.0"
short_description: Create, update, or manage Azure Monitor diagnostic settings.

description:
    - Create, update, or manage Azure Monitor diagnostic settings for any type of resource.

options:
    name:
        description:
            - The name of the diagnostic settings.
        type: str
        required: true
    resource:
        description:
            - The resource which will be monitored with the diagnostic setting.
            - It can be a string containing the resource ID.
            - It can be a dictionary containing I(name), I(type), I(resource_group), and optionally I(subscription_id).
            - I(name). The resource name.
            - I(type). The resource type including namespace, such as 'Microsoft.Network/virtualNetworks'.
            - I(resource_group). The resource group containing the resource.
            - I(subscription_id). The subscription ID containing the resource. If none is specified, the credential's subscription ID will be used.
        type: raw
        required: true
    storage_account:
        description:
            - A storage account which will receive the diagnostic logs.
            - It can be a string containing the storage account resource ID.
            - It can be a dictionary containing I(name) and optionally I(subscription_id) and I(resource_group).
            - At least one of I(storage_account), I(log_analytics), or I(event_hub) must be specified for the diagnostic setting.
        type: raw
    log_analytics:
        description:
            - A log analytics workspace which will receive the diagnostic logs.
            - It can be a string containing the log analytics workspace resource ID.
            - It can be a dictionary containing I(name) and optionally I(subscription_id) and I(resource_group).
            - At least one of I(storage_account), I(log_analytics), or I(event_hub) must be specified for the diagnostic setting.
        type: raw
    event_hub:
        description:
            - An event hub which will receive the diagnostic logs.
            - At least one of I(storage_account), I(log_analytics), or I(event_hub) must be specified for the diagnostic setting.
        type: dict
        suboptions:
            namespace:
                description:
                    - The event hub namespace.
                type: str
                required: true
            policy:
                description:
                    - The shared access policy.
                type: str
                required: true
            hub:
                description:
                    - An event hub name to receive logs. If none is specified, the default event hub will be selected.
                type: str
            resource_group:
                description:
                    - The resource group containing the event hub. If none is specified, the resource group of the I(resource) parameter will be used.
                type: str
            subscription_id:
                description:
                    - The subscription ID containing the event hub. If none is specified, the subscription ID of the I(resource) parameter will be used.
                type: str
    logs:
        description:
            - The list of log setttings.
            - At least one of I(metrics) or I(logs) must be specified for the diagnostic setting.
        type: list
        elements: dict
        suboptions:
            category:
                description:
                    - Name of a Management Group Diagnostic Log category for a resource type this setting is applied to.
                type: str
            category_group:
                description:
                    - Name of a Management Group Diagnostic Log category group for a resource type this setting is applied to.
                type: str
            enabled:
                description:
                    - Whether the log is enabled.
                type: bool
                default: true
            retention_policy:
                description:
                    - The retention policy for this log.
                type: dict
                suboptions:
                    days:
                        description:
                            - The number of days for the retention policy.
                        type: int
                        default: 0
                    enabled:
                        description:
                            - Whether the retention policy is enabled.
                        type: bool
                        default: true
    metrics:
        description:
            - The list of metric setttings.
            - At least one of I(metrics) or I(logs) must be specified for the diagnostic setting.
        type: list
        elements: dict
        suboptions:
            category:
                description:
                    - Name of a Diagnostic Metric category for a resource type this setting is applied to.
                type: str
            enabled:
                description:
                    - Whether the metric category is enabled.
                type: bool
                default: true
            retention_policy:
                description:
                    - The retention policy for this metric.
                type: dict
                suboptions:
                    days:
                        description:
                            - The number of days for the retention policy.
                        type: int
                        default: 0
                    enabled:
                        description:
                            - Whether the retention policy is enabled.
                        type: bool
                        default: true
    state:
        description:
            - State of the private endpoint DNS zone group. Use C(present) to create or update and C(absent) to delete.
        default: present
        type: str
        choices:
            - absent
            - present

extends_documentation_fragment:
    - azure.azcollection.azure

author:
    - Ross Bender (@l3ender)
'''

EXAMPLES = '''
- name: Create storage-based diagnostic setting for a virtual network
  azure_rm_monitordiagnosticsetting:
    name: "logs-storage"
    resource: "{{ vnet_output.state.id }}"
    storage_account: "{{ storage_output.state.id }}"
    logs:
      - category_group: "allLogs"
    metrics:
      - category: "AllMetrics"

- name: Create diagnostic setting for webapp with log analytics, event hub, and storage
  azure_rm_monitordiagnosticsetting:
    name: "webapp-logs"
    resource:
      name: "my-webapp"
      type: "Microsoft.Web/sites"
      resource_group: "my-webapp-resource-group"
    event_hub:
      namespace: "my-event-hub"
      policy: "RootManageSharedAccessKey"
    log_analytics:
      name: "my-log-analytics-workspace"
      resource_group: "my-log-analytics-workspace-resource-group"
    storage_account:
      name: "mystorageaccount"
    logs:
      - category: "AppServiceHTTPLogs"
      - category: "AppServiceConsoleLogs"
      - category: "AppServiceAppLogs"
      - category: "AppServiceAuditLogs"
      - category: "AppServiceIPSecAuditLogs"
      - category: "AppServicePlatformLogs"

- name: Delete diagnostic setting
  azure_rm_monitordiagnosticsetting:
    name: "webapp-logs"
    resource:
      name: "my-webapp"
      type: "Microsoft.Web/sites"
      resource_group: "my-webapp-resource-group"
    state: "absent"
'''

RETURN = '''
state:
    description:
        - The state of the diagnostic setting.
    returned: always
    type: dict
    contains:
        id:
            description:
                - ID of the diagnostic setting.
            sample: >-
                /subscriptions/xxx/resourcegroups/my-resource-group/providers/microsoft.network/applicationgateways/my-appgw/
                providers/microsoft.insights/diagnosticSettings/my-diagnostic-setting
            returned: always
            type: str
        name:
            description:
                - Name of the diagnostic setting.
            returned: always
            type: str
            sample: my-diagnostic-setting
        logs:
            description:
                - Enabled log configurations for the diagnostic setting.
            returned: always
            type: list
            elements: dict
            contains:
                category:
                    description:
                        - Name of a Management Group Diagnostic Log category for a resource type this setting is applied to.
                    type: str
                    returned: always
                category_group:
                    description:
                        - Name of a Management Group Diagnostic Log category group for a resource type this setting is applied to.
                    type: str
                    returned: always
                enabled:
                    description:
                        - Whether this log is enabled.
                    type: bool
                    returned: always
                retention_policy:
                    description:
                        - The retention policy for this log.
                    type: dict
                    returned: always
                    contains:
                        enabled:
                            description:
                                - Whether the retention policy is enabled.
                            type: bool
                            returned: always
                        days:
                            description:
                                - The number of days for the retention policy.
                            type: int
                            returned: always
        metrics:
            description:
                - Enabled metric configurations for the diagnostic setting.
            returned: always
            type: list
            elements: dict
            contains:
                category:
                    description:
                        - Name of a Diagnostic Metric category for a resource type this setting is applied to.
                    type: str
                    returned: always
                enabled:
                    description:
                        - Whether the metric category is enabled.
                    type: bool
                    returned: always
                retention_policy:
                    description:
                        - The retention policy for the metric category.
                    type: dict
                    returned: always
                    contains:
                        enabled:
                            description:
                                - Whether the retention policy is enabled.
                            type: bool
                            returned: always
                        days:
                            description:
                                - The number of days for the retention policy.
                            type: int
                            returned: always
        event_hub:
            description:
                - The event hub for the diagnostic setting, if configured.
            returned: always
            type: dict
            contains:
                id:
                    description:
                        - ID of the event hub namespace.
                    returned: always
                    type: str
                    sample: >-
                        /subscriptions/xxx/resourceGroups/my-resource-group/providers/Microsoft.EventHub/namespaces/my-event-hub-namespace
                namespace:
                    description:
                        - Name of the event hub namespace.
                    returned: always
                    type: str
                    sample: my-event-hub-namespace
                hub:
                    description:
                        - Name of the hub within the namespace.
                    returned: always
                    type: str
                    sample: my-event-hub
                policy:
                    description:
                        - Name of the event hub shared access policy.
                    returned: always
                    type: str
                    sample: RootManageSharedAccessKey
        log_analytics:
            description:
                - The log analytics workspace for the diagnostic setting, if configured.
            returned: always
            type: dict
            contains:
                id:
                    description:
                        - ID of the log analytics workspace.
                    returned: always
                    type: str
                    sample: >-
                        /subscriptions/xxx/resourcegroups/my-resource-group/providers/microsoft.operationalinsights/workspaces/my-log-analytics-workspace
        storage_account:
            description:
                - The storage account for the diagnostic setting, if configured.
            returned: always
            type: dict
            contains:
                id:
                    description:
                        - ID of the storage account.
                    returned: always
                    type: str
                    sample: >-
                        /subscriptions/xxx/resourceGroups/my-resource-group/providers/Microsoft.Storage/storageAccounts/my-storage-account
'''

try:
    from azure.mgmt.core.tools import (parse_resource_id, resource_id)
    from azure.core.polling import LROPoller
except ImportError:
    # This is handled in azure_rm_common
    pass

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common_ext import AzureRMModuleBaseExt


event_hub_spec = dict(
    namespace=dict(type="str", required=True),
    policy=dict(type="str", required=True),
    hub=dict(type="str"),
    resource_group=dict(type="str"),
    subscription_id=dict(type="str"),
)

retention_policy_spec = dict(
    days=dict(type="int", default=0),
    enabled=dict(type="bool", default=True),
)

logs_spec = dict(
    category=dict(type="str"),
    category_group=dict(type="str"),
    enabled=dict(type="bool", default=True),
    retention_policy=dict(type="dict", options=retention_policy_spec),
)

metrics_spec = dict(
    category=dict(type="str"),
    enabled=dict(type="bool", default=True),
    retention_policy=dict(type="dict", options=retention_policy_spec),
)


class Actions:
    NoAction, Create, Update, Delete = range(4)


class AzureRMMonitorDiagnosticSetting(AzureRMModuleBaseExt):

    def __init__(self):

        self.module_arg_spec = dict(
            name=dict(type="str", required=True),
            resource=dict(type="raw", required=True),
            storage_account=dict(type="raw"),
            log_analytics=dict(type="raw"),
            event_hub=dict(type="dict", options=event_hub_spec),
            logs=dict(type="list", elements="dict", options=logs_spec),
            metrics=dict(type="list", elements="dict", options=metrics_spec),
            state=dict(type="str", default="present", choices=["present", "absent"]),
        )

        self.name = None
        self.resource = None
        self.state = None
        self.parameters = dict()
        self.results = dict(
            changed=False,
            state=dict()
        )
        self.to_do = Actions.NoAction

        super(AzureRMMonitorDiagnosticSetting, self).__init__(self.module_arg_spec,
                                                              required_if=[
                                                                  ("state", "present", ("storage_account", "log_analytics", "event_hub"), True),
                                                                  ("state", "present", ("logs", "metrics"), True),
                                                              ],
                                                              supports_tags=False,
                                                              supports_check_mode=True)

    def exec_module(self, **kwargs):
        for key in self.module_arg_spec:
            if hasattr(self, key):
                setattr(self, key, kwargs[key])
            elif kwargs[key] is not None:
                self.parameters[key] = kwargs[key]

        self.process_parameters()

        old_response = self.get_item()

        if old_response is None or not old_response:
            if self.state == "present":
                self.to_do = Actions.Create
        else:
            if self.state == "absent":
                self.to_do = Actions.Delete
            else:
                self.results["compare"] = []
                if not self.idempotency_check(old_response, self.diagnostic_setting_to_dict(self.parameters)):
                    self.to_do = Actions.Update

        if (self.to_do == Actions.Create) or (self.to_do == Actions.Update):
            self.results["changed"] = True
            if self.check_mode:
                return self.results
            response = self.create_update_setting()
        elif self.to_do == Actions.Delete:
            self.results["changed"] = True
            if self.check_mode:
                return self.results
            response = self.delete_setting()
        else:
            self.results["changed"] = False
            response = old_response

        if response is not None:
            self.results["state"] = response

        return self.results

    def process_parameters(self):
        if isinstance(self.resource, dict):
            if "/" not in self.resource.get("type"):
                self.fail("resource type parameter must include namespace, such as 'Microsoft.Network/virtualNetworks'")
            self.resource = resource_id(subscription=self.resource.get("subscription_id", self.subscription_id),
                                        resource_group=self.resource.get("resource_group"),
                                        namespace=self.resource.get("type").split("/")[0],
                                        type=self.resource.get("type").split("/")[1],
                                        name=self.resource.get("name"))

        parsed_resource = parse_resource_id(self.resource)

        storage_account = self.parameters.pop("storage_account", None)
        if storage_account:
            if isinstance(storage_account, dict):
                if not storage_account.get("name"):
                    self.fail("storage_account must contain 'name'")

                storage_account_id = resource_id(subscription=storage_account.get("subscription_id", parsed_resource.get("subscription")),
                                                 resource_group=storage_account.get("resource_group", parsed_resource.get("resource_group")),
                                                 namespace="Microsoft.Storage",
                                                 type="storageAccounts",
                                                 name=storage_account.get("name"))
            else:
                storage_account_id = storage_account

            self.parameters["storage_account_id"] = storage_account_id

        log_analytics = self.parameters.pop("log_analytics", None)
        if log_analytics:
            if isinstance(log_analytics, dict):
                if not log_analytics.get("name"):
                    self.fail("log_analytics must contain 'name'")

                log_analytics_id = resource_id(subscription=log_analytics.get("subscription_id", parsed_resource.get("subscription")),
                                               resource_group=log_analytics.get("resource_group", parsed_resource.get("resource_group")),
                                               namespace="microsoft.operationalinsights",
                                               type="workspaces",
                                               name=log_analytics.get("name"))
            else:
                log_analytics_id = log_analytics

            self.parameters["workspace_id"] = log_analytics_id

        event_hub = self.parameters.pop("event_hub", None)
        if event_hub:
            hub_subscription_id = event_hub.get("subscription_id") if event_hub.get("subscription_id") else parsed_resource.get("subscription")
            hub_resource_group = event_hub.get("resource_group") if event_hub.get("resource_group") else parsed_resource.get("resource_group")
            auth_rule_id = resource_id(subscription=hub_subscription_id,
                                       resource_group=hub_resource_group,
                                       namespace="Microsoft.EventHub",
                                       type="namespaces",
                                       name=event_hub.get("namespace"),
                                       child_type_1="authorizationrules",
                                       child_name_1=event_hub.get("policy"))
            self.parameters["event_hub_authorization_rule_id"] = auth_rule_id
            self.parameters["event_hub_name"] = event_hub.get("hub")

    def get_item(self):
        self.log("Get diagnostic setting for {0} in {1}".format(self.name, self.resource))

        try:
            item = self.monitor_diagnostic_settings_client.diagnostic_settings.get(resource_uri=self.resource, name=self.name)
            return self.diagnostic_setting_to_dict(item)
        except Exception:
            self.log("Did not find diagnostic setting for {0} in {1}".format(self.name, self.resource))

        return None

    def create_update_setting(self):
        try:
            response = self.monitor_diagnostic_settings_client.diagnostic_settings.create_or_update(resource_uri=self.resource,
                                                                                                    name=self.name,
                                                                                                    parameters=self.parameters)
            if isinstance(response, LROPoller):
                response = self.get_poller_result(response)

            return self.diagnostic_setting_to_dict(response)
        except Exception as exc:
            self.fail("Error creating or updating diagnostic setting {0} for resource {1}: {2}".format(self.name, self.resource, str(exc)))

    def delete_setting(self):
        try:
            response = self.monitor_diagnostic_settings_client.diagnostic_settings.delete(resource_uri=self.resource,
                                                                                          name=self.name)
            if isinstance(response, LROPoller):
                response = self.get_poller_result(response)

            return response
        except Exception as exc:
            self.fail("Error deleting diagnostic setting {0} for resource {1}: {2}".format(self.name, self.resource, str(exc)))

    def diagnostic_setting_to_dict(self, diagnostic_setting):
        setting_dict = diagnostic_setting if isinstance(diagnostic_setting, dict) else diagnostic_setting.as_dict()
        result = dict(
            id=setting_dict.get("id"),
            name=setting_dict.get("name"),
            event_hub=self.event_hub_dict(setting_dict),
            storage_account=self.storage_dict(setting_dict.get("storage_account_id")),
            log_analytics=self.log_analytics_dict(setting_dict.get("workspace_id")),
            logs=[self.log_config_to_dict(log) for log in setting_dict.get("logs", [])],
            metrics=[self.metric_config_to_dict(metric) for metric in setting_dict.get("metrics", [])],
        )
        return self.remove_disabled_config(result)

    def remove_disabled_config(self, diagnostic_setting):
        diagnostic_setting["logs"] = [log for log in diagnostic_setting.get("logs", []) if log.get("enabled")]
        diagnostic_setting["metrics"] = [metric for metric in diagnostic_setting.get("metrics", []) if metric.get("enabled")]
        return diagnostic_setting

    def event_hub_dict(self, setting_dict):
        auth_rule_id = setting_dict.get("event_hub_authorization_rule_id")
        if auth_rule_id:
            parsed_rule_id = parse_resource_id(auth_rule_id)
            return dict(
                id=resource_id(subscription=parsed_rule_id.get("subscription"),
                               resource_group=parsed_rule_id.get("resource_group"),
                               namespace=parsed_rule_id.get("namespace"),
                               type=parsed_rule_id.get("type"),
                               name=parsed_rule_id.get("name")),
                namespace=parsed_rule_id.get("name"),
                hub=setting_dict.get("event_hub_name"),
                policy=parsed_rule_id.get("resource_name"),
            )
        return None

    def storage_dict(self, storage_account_id):
        if storage_account_id:
            return dict(
                id=storage_account_id,
            )
        return None

    def log_analytics_dict(self, workspace_id):
        if workspace_id:
            return dict(
                id=workspace_id,
            )
        return None

    def log_config_to_dict(self, log_config):
        return dict(
            category=log_config.get("category"),
            category_group=log_config.get("category_group"),
            enabled=log_config.get("enabled"),
            retention_policy=self.retention_policy_to_dict(log_config.get("retention_policy")),
        )

    def metric_config_to_dict(self, metric_config):
        return dict(
            category=metric_config.get("category"),
            enabled=metric_config.get("enabled"),
            retention_policy=self.retention_policy_to_dict(metric_config.get("retention_policy")),
        )

    def retention_policy_to_dict(self, policy):
        if policy:
            return dict(
                days=policy.get("days"),
                enabled=policy.get("enabled"),
            )
        return None


def main():
    AzureRMMonitorDiagnosticSetting()


if __name__ == "__main__":
    main()



#!/usr/bin/python
#
# This file is part of Ansible
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#
from __future__ import absolute_import, division, print_function


__metaclass__ = type
DOCUMENTATION = """
module: ios_facts
author:
  - Peter Sprygada (@privateip)
  - Sumit Jaiswal (@justjais)
short_description: Module to collect facts from remote devices.
description:
  - Collects a base set of device facts from a remote device that is running IOS.  This
    module prepends all of the base network fact keys with C(ansible_net_<fact>).  The
    facts module will always collect a base set of facts from the device and can enable
    or disable collection of additional facts.
version_added: 1.0.0
extends_documentation_fragment:
  - cisco.ios.ios
notes:
  - Tested against Cisco IOSXE Version 17.3 on CML and IOS 15.6 for L2 specific resource.
  - Facts gathering for L3 devices are supposed to produce blank output for unsupported
    resources like vlan.
  - This module works with connection C(network_cli).
    See U(https://docs.ansible.com/ansible/latest/network/user_guide/platform_ios.html)
options:
  gather_subset:
    description:
      - When supplied, this argument restricts the facts collected to a given subset.
      - Possible values for this argument include C(all), C(min), C(default), C(hardware), C(config),
        and C(interfaces).
      - Specify a list of values to include a larger subset.
      - Use a value with an initial C(!) to collect all facts except that subset.
    required: false
    default: "min"
    type: list
    elements: str
  gather_network_resources:
    description:
      - When supplied, this argument will restrict the facts collected to a given subset.
        Possible values for this argument include all and the resources like interfaces,
        vlans etc. Can specify a list of values to include a larger subset. Values can
        also be used with an initial C(!) to specify that a specific subset should
        not be collected. Valid subsets are 'bgp_global', 'l3_interfaces', 'lag_interfaces',
        'ntp_global', 'acls', 'hostname', 'interfaces', 'lldp_interfaces', 'logging_global',
        'ospf_interfaces', 'ospfv2', 'prefix_lists', 'static_routes', 'acl_interfaces',
        'all', 'bgp_address_family', 'l2_interfaces', 'lacp', 'lacp_interfaces', 'lldp_global',
        'ospfv3', 'snmp_server', 'vlans', 'service'.
    type: list
    elements: str
  available_network_resources:
    description: When 'True' a list of network resources for which resource modules are available will be provided.
    type: bool
    default: false
"""

EXAMPLES = """
- name: Gather all legacy facts
  cisco.ios.ios_facts:
    gather_subset: all

- name: Gather only the config and default facts
  cisco.ios.ios_facts:
    gather_subset:
      - config

- name: Do not gather hardware facts
  cisco.ios.ios_facts:
    gather_subset:
      - "!hardware"

- name: Gather legacy and resource facts
  cisco.ios.ios_facts:
    gather_subset: all
    gather_network_resources: all

- name: Gather only the interfaces resource facts and no legacy facts
  cisco.ios.ios_facts:
    gather_subset:
      - "!all"
      - "!min"
    gather_network_resources:
      - interfaces

- name: Gather interfaces resource and minimal legacy facts
  cisco.ios.ios_facts:
    gather_subset: min
    gather_network_resources: interfaces

- name: Gather L2 interfaces resource and minimal legacy facts
  cisco.ios.ios_facts:
    gather_subset: min
    gather_network_resources: l2_interfaces

- name: Gather L3 interfaces resource and minimal legacy facts
  cisco.ios.ios_facts:
    gather_subset: min
    gather_network_resources: l3_interfaces
"""

RETURN = """
ansible_net_gather_subset:
  description: The list of fact subsets collected from the device
  returned: always
  type: list

ansible_net_gather_network_resources:
  description: The list of fact for network resource subsets collected from the device
  returned: when the resource is configured
  type: list

# default
ansible_net_model:
  description: The model name returned from the device
  returned: always
  type: str
ansible_net_serialnum:
  description: The serial number of the remote device
  returned: always
  type: str
ansible_net_version:
  description: The operating system version running on the remote device
  returned: always
  type: str
ansible_net_iostype:
  description: The operating system type (IOS or IOS-XE) running on the remote device
  returned: always
  type: str
ansible_net_hostname:
  description: The configured hostname of the device
  returned: always
  type: str
ansible_net_image:
  description: The image file the device is running
  returned: always
  type: str
ansible_net_stacked_models:
  description: The model names of each device in the stack
  returned: when multiple devices are configured in a stack
  type: list
ansible_net_stacked_serialnums:
  description: The serial numbers of each device in the stack
  returned: when multiple devices are configured in a stack
  type: list
ansible_net_api:
  description: The name of the transport
  returned: always
  type: str
ansible_net_python_version:
  description: The Python version Ansible controller is using
  returned: always
  type: str

# hardware
ansible_net_filesystems:
  description: All file system names available on the device
  returned: when hardware is configured
  type: list
ansible_net_filesystems_info:
  description: A hash of all file systems containing info about each file system (e.g. free and total space)
  returned: when hardware is configured
  type: dict
ansible_net_memfree_mb:
  description: The available free memory on the remote device in MiB
  returned: when hardware is configured
  type: int
ansible_net_memtotal_mb:
  description: The total memory on the remote device in MiB
  returned: when hardware is configured
  type: int
ansible_net_cpu_utilization:
  description: The current CPU utilization of the device
  returned: when hardware is configured
  type: dict

# config
ansible_net_config:
  description: The current active config from the device
  returned: when config is configured
  type: str

# interfaces
ansible_net_all_ipv4_addresses:
  description: All IPv4 addresses configured on the device
  returned: when interfaces is configured
  type: list
ansible_net_all_ipv6_addresses:
  description: All IPv6 addresses configured on the device
  returned: when interfaces is configured
  type: list
ansible_net_interfaces:
  description: A hash of all interfaces running on the system
  returned: when interfaces is configured
  type: dict
ansible_net_neighbors:
  description:
    - The list of CDP and LLDP neighbors from the remote device. If both,
      CDP and LLDP neighbor data is present on one port, CDP is preferred.
  returned: when interfaces is configured
  type: dict
"""
from ansible.module_utils.basic import AnsibleModule

from ansible_collections.cisco.ios.plugins.module_utils.network.ios.argspec.facts.facts import (
    FactsArgs,
)
from ansible_collections.cisco.ios.plugins.module_utils.network.ios.facts.facts import (
    FACT_RESOURCE_SUBSETS,
    Facts,
)


def main():
    """
    Main entry point for module execution

    :returns: ansible_facts
    """
    argument_spec = FactsArgs.argument_spec
    module = AnsibleModule(argument_spec=argument_spec, supports_check_mode=True)
    warnings = []

    ansible_facts = {}
    if module.params.get("available_network_resources"):
        ansible_facts["available_network_resources"] = sorted(FACT_RESOURCE_SUBSETS.keys())
    result = Facts(module).get_facts()
    additional_facts, additional_warnings = result
    ansible_facts.update(additional_facts)
    warnings.extend(additional_warnings)
    module.exit_json(ansible_facts=ansible_facts, warnings=warnings)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: device_health_v1_info
short_description: Information module for Device Health V1
description:
- Get all Device Health V1.
- >
   Intent API for accessing DNA Assurance Device object for generating reports, creating dashboards or creating
   additional value added services.
version_added: '3.1.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
  deviceRole:
    description:
    - DeviceRole query parameter. CORE, ACCESS, DISTRIBUTION, ROUTER, WLC, or AP (case insensitive).
    type: str
  siteId:
    description:
    - SiteId query parameter. DNAC site UUID.
    type: str
  health:
    description:
    - Health query parameter. DNAC health catagory POOR, FAIR, or GOOD (case insensitive).
    type: str
  startTime:
    description:
    - StartTime query parameter. UTC epoch time in milliseconds.
    type: float
  endTime:
    description:
    - EndTime query parameter. UTC epoch time in milliseconds.
    type: float
  limit:
    description:
    - Limit query parameter. Max number of device entries in the response (default to 50. Max at 500).
    type: float
  offset:
    description:
    - Offset query parameter. The offset of the first device in the returned data (Mutiple of 'limit' + 1).
    type: float
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Devices DevicesV1
  description: Complete reference of the DevicesV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!devices
notes:
  - SDK Method used are
    devices.Devices.devices_v1,

  - Paths used are
    get /dna/intent/api/v1/device-health,

"""

EXAMPLES = r"""
- name: Get all Device Health V1
  cisco.dnac.device_health_v1_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    deviceRole: string
    siteId: string
    health: string
    startTime: 0
    endTime: 0
    limit: 0
    offset: 0
  register: result

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "version": "string",
      "totalCount": 0,
      "response": [
        {
          "deviceType": "string",
          "cpuUtilization": 0,
          "overallHealth": 0,
          "utilizationHealth": {
            "radio0": 0,
            "radio1": 0,
            "radio2": 0,
            "radio3": 0,
            "Ghz24": 0,
            "Ghz50": 0
          },
          "airQualityHealth": {
            "radio0": 0,
            "radio1": 0,
            "radio2": 0,
            "radio3": 0,
            "Ghz24": 0,
            "Ghz50": 0
          },
          "ipAddress": "string",
          "cpuHealth": 0,
          "deviceFamily": "string",
          "issueCount": 0,
          "macAddress": "string",
          "noiseHealth": {
            "radio0": 0,
            "radio1": 0,
            "radio2": 0,
            "radio3": 0,
            "Ghz24": 0,
            "Ghz50": 0
          },
          "osVersion": "string",
          "name": "string",
          "interfaceLinkErrHealth": 0,
          "memoryUtilization": 0,
          "interDeviceLinkAvailHealth": 0,
          "interferenceHealth": {
            "radio0": 0,
            "radio1": 0,
            "radio2": 0,
            "radio3": 0,
            "Ghz24": 0,
            "Ghz50": 0
          },
          "model": "string",
          "location": "string",
          "reachabilityHealth": "string",
          "band": {
            "radio0": "string",
            "radio1": "string",
            "radio2": "string",
            "radio3": 0
          },
          "memoryUtilizationHealth": 0,
          "clientCount": {
            "radio0": 0,
            "radio1": 0,
            "radio2": 0,
            "radio3": 0,
            "Ghz24": 0,
            "Ghz50": 0
          },
          "avgTemperature": 0,
          "maxTemperature": 0,
          "interDeviceLinkAvailFabric": 0,
          "apCount": 0,
          "freeTimerScore": 0,
          "freeTimer": 0,
          "packetPoolHealth": 0,
          "packetPool": 0,
          "freeMemoryBufferHealth": 0,
          "freeMemoryBuffer": 0,
          "wqePoolsHealth": 0,
          "wqePools": 0,
          "wanLinkUtilization": 0,
          "cpuUlitilization": 0,
          "uuid": "string"
        }
      ]
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)


DOCUMENTATION = r"""
---
module: sda_fabrics_vlan_to_ssids_fabric_id
short_description: Resource module for Sda Fabrics Vlan To Ssids Fabric Id
description:
- This module represents an alias of the module sda_fabrics_vlan_to_ssids_fabric_id_v1
version_added: '6.15.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  fabricId:
    description: FabricId path parameter. The 'fabricId' represents the Fabric ID of
      a particular Fabric Site.
    type: str
  payload:
    description: Sda Fabrics Vlan To Ssids Fabric Id's payload.
    elements: dict
    suboptions:
      ssidDetails:
        description: Sda Fabrics Vlan To Ssids Fabric Id's ssidDetails.
        elements: dict
        suboptions:
          name:
            description: Name of the SSID.
            type: str
          securityGroupTag:
            description: Represents the name of the Security Group. Example Auditors,
              BYOD, Developers, etc.
            type: str
        type: list
      vlanName:
        description: Vlan Name.
        type: str
    type: list
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Fabric Wireless AddUpdateOrRemoveSSIDMappingToAVLANV1
  description: Complete reference of the AddUpdateOrRemoveSSIDMappingToAVLANV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!add-update-or-remove-ssid-mapping-to-avlan
notes:
  - SDK Method used are
    fabric_wireless.FabricWireless.add_update_or_remove_ssid_mapping_to_a_vlan_v1,

  - Paths used are
    put /dna/intent/api/v1/sda/fabrics/{fabricId}/vlanToSsids,
  - It should be noted that this module is an alias of sda_fabrics_vlan_to_ssids_fabric_id_v1

"""

EXAMPLES = r"""
- name: Update all
  cisco.dnac.sda_fabrics_vlan_to_ssids_fabric_id:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: present
    fabricId: string
    payload:
    - ssidDetails:
      - name: string
        securityGroupTag: string
      vlanName: string

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": {
        "taskId": "string",
        "url": "string"
      },
      "version": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: patch_rollback
short_description: Resource module for Patch Rollback
description:
- Manage operation create of the resource Patch Rollback.
- >
   Triggers patch rollback on the Cisco ISE node. A task ID is returned which can be used to monitor the progress of the patch rollback process. As the
   patch rollback triggers the Cisco ISE to restart, the task API becomes unavailable for a certain period of time. For rollback of patch on secondary
   nodes , request needs to sent on individual node instead of the Primary Administration Node.
version_added: '2.1.0'
extends_documentation_fragment:
  - cisco.ise.module
author: Rafael Campos (@racampos)
options:
  patchNumber:
    description: Patch Rollback's patchNumber.
    type: int
requirements:
- ciscoisesdk >= 2.2.3
- python >= 3.5
seealso:
- name: Cisco ISE documentation for Patching
  description: Complete reference of the Patching API.
  link: https://developer.cisco.com/docs/identity-services-engine/v1/#!patch-and-hot-patch-openapi
notes:
  - SDK Method used are
    patching.Patching.rollback_patch,

  - Paths used are
    post /api/v1/patch/rollback,

"""

EXAMPLES = r"""
- name: Create
  cisco.ise.patch_rollback:
    ise_hostname: "{{ise_hostname}}"
    ise_username: "{{ise_username}}"
    ise_password: "{{ise_password}}"
    ise_verify: "{{ise_verify}}"
    patchNumber: 0

"""

RETURN = r"""
ise_response:
  description: A dictionary or list with the response returned by the Cisco ISE Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": {
        "id": "string",
        "message": "string"
      },
      "version": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright 2021 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

"""
The module file for eos_ntp_global
"""

from __future__ import absolute_import, division, print_function


__metaclass__ = type

DOCUMENTATION = """
---
module: eos_ntp_global
short_description: Manages ntp resource module
description: This module configures and manages the attributes of  ntp on Arista
  EOS platforms.
version_added: 3.1.0
author: Gomathi Selvi Srinivasan (@GomathiselviS)
notes:
- Tested against Arista EOS 4.24.60M
- This module works with connection C(network_cli). See the U(https://docs.ansible.com/ansible/latest/network/user_guide/platform_eos.html).
options:
    config:
      description: A dictionary of ntp options
      type: dict
      suboptions:
        authenticate:
          description:
          - Require authentication for NTP synchronization.
          type: dict
          suboptions:
            enable:
              description: Enable authentication for NTP synchronization.
              type: bool
            servers:
              description: Authentication required only for incoming NTP server responses.
              type: bool
        authentication_keys:
          description:
          - Define a key to use for authentication.
          type: list
          elements: dict
          suboptions:
            id:
              description: key identifier.
              type: int
            algorithm:
              description: hash algorithm,
              type: str
              choices: ["md5", "sha1"]
            encryption:
              description: key type
              type: int
              choices: [0, 7]
            key:
              description: Unobfuscated key string.
              type: str
        local_interface:
          description: Configure the interface from which the IP source address is taken.
          type: str
        qos_dscp:
          description: Set DSCP value in IP header
          type: int
        serve:
          description: Configure the switch as an NTP server.
          type: dict
          suboptions:
            all:
              description: Service NTP requests received on any interface.
              type: bool
            access_lists:
              description: Configure access control list.
              type: list
              elements: dict
              suboptions:
                afi:
                  description: ip/ipv6 config commands.
                  type: str
                acls:
                  description: Access lists to be configured under the afi
                  type: list
                  elements: dict
                  suboptions:
                    acl_name:
                      description: Name of the access list.
                      type: str
                    direction:
                      description: direction for the packets.
                      type: str
                      choices: ["in", "out"]
                    vrf:
                      description: VRF in which to apply the access control list.
                      type: str
        servers:
          description: Configure NTP server to synchronize to.
          type: list
          elements: dict
          suboptions:
            vrf:
              description: vrf name.
              type: str
            server:
              description: Hostname or A.B.C.D or A:B:C:D:E:F:G:H.
              type: str
              required: true
            burst:
              description: Send a burst of packets instead of the usual one.
              type: bool
            iburst:
              description: Send bursts of packets until the server is reached
              type: bool
            key_id:
              description: Set a key to use for authentication.
              type: int
            local_interface:
              description: Configure the interface from which the IP source address is taken.
              type: str
            source:
              description: Configure the interface from which the IP source address is taken.
              type: str
            maxpoll:
              description: Maximum poll interval.
              type: int
            minpoll:
              description: Minimum poll interval.
              type: int
            prefer:
              description: Mark this server as preferred.
              type: bool
            version:
              description: NTP version.
              type: int
        trusted_key:
          description: Configure the set of keys that are accepted for incoming messages
          type: str
    running_config:
      description:
      - This option is used only with state I(parsed).
      - The value of this option should be the output received from the EOS device by
        executing the command B(show running-config | section ntp).
      - The state I(parsed) reads the configuration from C(running_config) option and
        transforms it into Ansible structured data as per the resource module's argspec
        and the value is then returned in the I(parsed) key within the result.
      type: str
    state:
      description:
        - The state the configuration should be left in.
        - The states I(replaced) and I(overridden) have identical
          behaviour for this module.
        - Please refer to examples for more details.
      type: str
      choices:
      - deleted
      - merged
      - overridden
      - replaced
      - gathered
      - rendered
      - parsed
      default: merged
"""

EXAMPLES = """
# Using merged

# Before state:
# -------------
# localhost(config)#show running-config | section ntp
# localhost(config)#

- name: Merge provided configuration with device configuration
  arista.eos.eos_ntp_global:
    config:
      authenticate:
        enable: true
      authentication_keys:
        - id: 2
          algorithm: "sha1"
          encryption: 7
          key: "123456"
        - id: 23
          algorithm: "md5"
          encryption: 7
          key: "123456"
      local_interface: "Ethernet1"
      qos_dscp: 10
      trusted_key: 23
      servers:
        - server: "10.1.1.1"
          vrf: "vrf01"
          burst: true
          prefer: true
        - server: "25.1.1.1"
          vrf: "vrf01"
          maxpoll: 15
          key_id: 2
      serve:
        access_lists:
          - afi: "ip"
            acls:
              - acl_name: "acl01"
                direction: "in"
          - afi: "ipv6"
            acls:
              - acl_name: "acl02"
                direction: "in"

# After State

# localhost(config)#show running-config | section ntp
# ntp authentication-key 2 sha1 7 123456
# ntp authentication-key 23 md5 7 123456
# ntp trusted-key 23
# ntp authenticate
# ntp local-interface Ethernet1
# ntp qos dscp 10
# ntp server vrf vrf01 10.1.1.1 prefer burst
# ntp server vrf vrf01 25.1.1.1 maxpoll 15 key 2
# ntp serve ip access-group acl01 in
# ntp serve ipv6 access-group acl02 in
# localhost(config)#
#
#
# Module Execution:
# "after": {
#         "authenticate": {
#             "enable": true
#         },
#         "authentication_keys": [
#             {
#                 "algorithm": "sha1",
#                 "encryption": 7,
#                 "id": 2,
#                 "key": "VALUE_SPECIFIED_IN_NO_LOG_PARAMETER"
#             },
#             {
#                 "algorithm": "md5",
#                 "encryption": 7,
#                 "id": 23,
#                 "key": "VALUE_SPECIFIED_IN_NO_LOG_PARAMETER"
#             }
#         ],
#         "local_interface": "Ethernet1",
#         "qos_dscp": 10,
#         "serve": {
#             "access_lists": [
#                 {
#                     "acls": [
#                         {
#                             "acl_name": "acl01",
#                             "direction": "in"
#                         }
#                     ],
#                     "afi": "ip"
#                 },
#                 {
#                     "acls": [
#                         {
#                             "acl_name": "acl02",
#                             "direction": "in"
#                         }
#                     ],
#                     "afi": "ipv6"
#                 }
#             ]
#         },
#         "servers": [
#             {
#                 "burst": true,
#                 "prefer": true,
#                 "server": "10.1.1.1",
#                 "vrf": "vrf01"
#             },
#             {
#                 "key_id": 2,
#                 "maxpoll": 15,
#                 "server": "25.1.1.1",
#                 "vrf": "vrf01"
#             }
#         ],
#         "trusted_key": "23"
#     },
#     "before": {},
#     "changed": true,
#     "commands": [
#         "ntp serve ip access-group acl01 in",
#         "ntp serve ipv6 access-group acl02 in",
#         "ntp authentication-key 2 sha1 7 ********",
#         "ntp authentication-key 23 md5 7 ********",
#         "ntp server vrf vrf01 10.1.1.1 burst prefer",
#         "ntp server vrf vrf01 25.1.1.1 key 2 maxpoll 15",
#         "ntp authenticate",
#         "ntp local-interface Ethernet1",
#         "ntp qos dscp 10",
#         "ntp trusted-key 23"
#     ],

# Using Replaced

# Before State

# localhost(config)#show running-config | section ntp
# ntp authentication-key 2 sha1 7 123456
# ntp authentication-key 23 md5 7 123456
# ntp trusted-key 23
# ntp authenticate
# ntp local-interface Ethernet1
# ntp qos dscp 10
# ntp server vrf vrf01 10.1.1.1 prefer burst
# ntp server vrf vrf01 25.1.1.1 maxpoll 15 key 2
# ntp serve ip access-group acl01 in
# ntp serve ipv6 access-group acl02 in
# localhost(config)#

- name: Replace
  arista.eos.eos_ntp_global:
    config:
      qos_dscp: 15
      authentication_keys:
        - id: 2
          algorithm: "md5"
          encryption: 7
          key: "123456"
      servers:
        - server: "11.21.1.1"
          vrf: "vrf01"
          burst: true
          prefer: true
          minpoll: 13
      serve:
        access_lists:
          - afi: "ip"
            acls:
              - acl_name: "acl03"
                direction: "in"
    state: replaced

# After State:
# localhost(config)#show running-config | section ntp
# ntp authentication-key 2 md5 7 123456
# ntp qos dscp 15
# ntp server vrf vrf01 11.21.1.1 prefer burst minpoll 13
# ntp serve ip access-group acl03 in
# localhost(config)#
#
#
# Module Execution:
# "after": {
#        "authentication_keys": [
#            {
#                "algorithm": "md5",
#                "encryption": 7,
#                "id": 2,
#                "key": "VALUE_SPECIFIED_IN_NO_LOG_PARAMETER"
#            }
#        ],
#        "qos_dscp": 15,
#        "serve": {
#            "access_lists": [
#                {
#                    "acls": [
#                        {
#                            "acl_name": "acl03",
#                            "direction": "in"
#                        }
#                    ],
#                    "afi": "ip"
#                }
#            ]
#        },
#        "servers": [
#            {
#                "burst": true,
#                "minpoll": 13,
#                "prefer": true,
#                "server": "11.21.1.1",
#                "vrf": "vrf01"
#            }
#        ]
#    },
#    "before": {
#        "authenticate": {
#            "enable": true
#        },
#        "authentication_keys": [
#            {
#                "algorithm": "sha1",
#                "encryption": 7,
#                "id": 2,
#                "key": "VALUE_SPECIFIED_IN_NO_LOG_PARAMETER"
#            },
#            {
#                "algorithm": "md5",
#                "encryption": 7,
#                "id": 23,
#                "key": "VALUE_SPECIFIED_IN_NO_LOG_PARAMETER"
#            }
#        ],
#        "local_interface": "Ethernet1",
#        "qos_dscp": 10,
#        "serve": {
#            "access_lists": [
#                {
#                    "acls": [
#                        {
#                            "acl_name": "acl01",
#                            "direction": "in"
#                        }
#                    ],
#                    "afi": "ip"
#                },
#                {
#                    "acls": [
#                        {
#                            "acl_name": "acl02",
#                            "direction": "in"
#                        }
#                    ],
#                    "afi": "ipv6"
#                }
#            ]
#        },
#        "servers": [
#            {
#                "burst": true,
#                "prefer": true,
#                "server": "10.1.1.1",
#                "vrf": "vrf01"
#            },
#            {
#                "key_id": 2,
#                "maxpoll": 15,
#                "server": "25.1.1.1",
#                "vrf": "vrf01"
#            }
#        ],
#        "trusted_key": "23"
#    },
#    "changed": true,
#    "commands": [
#        "no ntp serve ip access-group acl01 in",
#        "no ntp serve ipv6 access-group acl02 in",
#        "no ntp authentication-key 23 md5 7 ********",
#        "no ntp server vrf vrf01 10.1.1.1 burst prefer",
#        "no ntp server vrf vrf01 25.1.1.1 key 2 maxpoll 15",
#        "no ntp authenticate",
#        "no ntp local-interface Ethernet1",
#        "no ntp trusted-key 23",
#        "ntp serve ip access-group acl03 in",
#        "ntp authentication-key 2 md5 7 ********",
#        "ntp server vrf vrf01 11.21.1.1 burst minpoll 13 prefer",
#        "ntp qos dscp 15"
#    ],
#
# Using Overridden

# Before State

# localhost(config)#show running-config | section ntp
# ntp authentication-key 2 sha1 7 123456
# ntp authentication-key 23 md5 7 123456
# ntp trusted-key 23
# ntp authenticate
# ntp local-interface Ethernet1
# ntp qos dscp 10
# ntp server vrf vrf01 10.1.1.1 prefer burst
# ntp server vrf vrf01 25.1.1.1 maxpoll 15 key 2
# ntp serve ip access-group acl01 in
# ntp serve ipv6 access-group acl02 in
# localhost(config)#

- name: Replace
  arista.eos.eos_ntp_global:
    config:
      qos_dscp: 15
      authentication_keys:
        - id: 2
          algorithm: "md5"
          encryption: 7
          key: "123456"
      servers:
        - server: "11.21.1.1"
          vrf: "vrf01"
          burst: true
          prefer: true
          minpoll: 13
      serve:
        access_lists:
          - afi: "ip"
            acls:
              - acl_name: "acl03"
                direction: "in"
    state: overridden

# After State:
# localhost(config)#show running-config | section ntp
# ntp authentication-key 2 md5 7 123456
# ntp qos dscp 15
# ntp server vrf vrf01 11.21.1.1 prefer burst minpoll 13
# ntp serve ip access-group acl03 in
# localhost(config)#
#
#
# Module Execution:
# "after": {
#        "authentication_keys": [
#            {
#                "algorithm": "md5",
#                "encryption": 7,
#                "id": 2,
#                "key": "VALUE_SPECIFIED_IN_NO_LOG_PARAMETER"
#            }
#        ],
#        "qos_dscp": 15,
#        "serve": {
#            "access_lists": [
#                {
#                    "acls": [
#                        {
#                            "acl_name": "acl03",
#                            "direction": "in"
#                        }
#                    ],
#                    "afi": "ip"
#                }
#            ]
#        },
#        "servers": [
#            {
#                "burst": true,
#                "minpoll": 13,
#                "prefer": true,
#                "server": "11.21.1.1",
#                "vrf": "vrf01"
#            }
#        ]
#    },
#    "before": {
#        "authenticate": {
#            "enable": true
#        },
#        "authentication_keys": [
#            {
#                "algorithm": "sha1",
#                "encryption": 7,
#                "id": 2,
#                "key": "VALUE_SPECIFIED_IN_NO_LOG_PARAMETER"
#            },
#            {
#                "algorithm": "md5",
#                "encryption": 7,
#                "id": 23,
#                "key": "VALUE_SPECIFIED_IN_NO_LOG_PARAMETER"
#            }
#        ],
#        "local_interface": "Ethernet1",
#        "qos_dscp": 10,
#        "serve": {
#            "access_lists": [
#                {
#                    "acls": [
#                        {
#                            "acl_name": "acl01",
#                            "direction": "in"
#                        }
#                    ],
#                    "afi": "ip"
#                },
#                {
#                    "acls": [
#                        {
#                            "acl_name": "acl02",
#                            "direction": "in"
#                        }
#                    ],
#                    "afi": "ipv6"
#                }
#            ]
#        },
#        "servers": [
#            {
#                "burst": true,
#                "prefer": true,
#                "server": "10.1.1.1",
#                "vrf": "vrf01"
#            },
#            {
#                "key_id": 2,
#                "maxpoll": 15,
#                "server": "25.1.1.1",
#                "vrf": "vrf01"
#            }
#        ],
#        "trusted_key": "23"
#    },
#    "changed": true,
#    "commands": [
#        "no ntp serve ip access-group acl01 in",
#        "no ntp serve ipv6 access-group acl02 in",
#        "no ntp authentication-key 23 md5 7 ********",
#        "no ntp server vrf vrf01 10.1.1.1 burst prefer",
#        "no ntp server vrf vrf01 25.1.1.1 key 2 maxpoll 15",
#        "no ntp authenticate",
#        "no ntp local-interface Ethernet1",
#        "no ntp trusted-key 23",
#        "ntp serve ip access-group acl03 in",
#        "ntp authentication-key 2 md5 7 ********",
#        "ntp server vrf vrf01 11.21.1.1 burst minpoll 13 prefer",
#        "ntp qos dscp 15"
#    ],
#

# using deleted:
# Before State

# localhost(config)#show running-config | section ntp
# ntp authentication-key 2 sha1 7 123456
# ntp authentication-key 23 md5 7 123456
# ntp trusted-key 23
# ntp authenticate
# ntp local-interface Ethernet1
# ntp qos dscp 10
# ntp server vrf vrf01 10.1.1.1 prefer burst
# ntp server vrf vrf01 11.21.1.1 prefer burst minpoll 13
# ntp server vrf vrf01 25.1.1.1 maxpoll 15 key 2
# ntp serve ip access-group acl01 in
# ntp serve ipv6 access-group acl02 in
# localhost(config)#

- name: Delete  ntp-global
  arista.eos.eos_ntp_global:
    state: deleted

# After State:
#  localhost(config)#show running-config | section ntp
# localhost(config)#
#
#
# # Module Execution
# "after": {},
#     "before": {
#         "authenticate": {
#             "enable": true
#         },
#         "authentication_keys": [
#             {
#                 "algorithm": "sha1",
#                 "encryption": 7,
#                 "id": 2,
#                 "key": "VALUE_SPECIFIED_IN_NO_LOG_PARAMETER"
#             },
#             {
#                 "algorithm": "md5",
#                 "encryption": 7,
#                 "id": 23,
#                 "key": "VALUE_SPECIFIED_IN_NO_LOG_PARAMETER"
#             }
#         ],
#         "local_interface": "Ethernet1",
#         "qos_dscp": 10,
#         "serve": {
#             "access_lists": [
#                 {
#                     "acls": [
#                         {
#                             "acl_name": "acl01",
#                             "direction": "in"
#                         }
#                     ],
#                     "afi": "ip"
#                 },
#                 {
#                     "acls": [
#                         {
#                             "acl_name": "acl02",
#                             "direction": "in"
#                         }
#                     ],
#                     "afi": "ipv6"
#                 }
#             ]
#         },
#         "servers": [
#             {
#                 "burst": true,
#                 "prefer": true,
#                 "server": "10.1.1.1",
#                 "vrf": "vrf01"
#             },
#             {
#                 "burst": true,
#                 "minpoll": 13,
#                 "prefer": true,
#                 "server": "11.21.1.1",
#                 "vrf": "vrf01"
#             },
#             {
#                 "key": 2,
#                 "maxpoll": 15,
#                 "server": "25.1.1.1",
#                 "vrf": "vrf01"
#             }
#         ],
#         "trusted_key": "23"
#     },
#     "changed": true,
#     "commands": [
#         "no ntp serve ip access-group acl01 in",
#         "no ntp serve ipv6 access-group acl02 in",
#         "no ntp authentication-key 2 sha1 7 ********",
#         "no ntp authentication-key 23 md5 7 ********",
#         "no ntp server vrf vrf01 10.1.1.1 burst prefer",
#         "no ntp server vrf vrf01 11.21.1.1 burst minpoll 13 prefer",
#         "no ntp server vrf vrf01 25.1.1.1 key 2 maxpoll 15",
#         "no ntp authenticate",
#         "no ntp local-interface Ethernet1",
#         "no ntp qos dscp 10",
#         "no ntp trusted-key 23"
#     ],
#

# Using parsed:
# parsed.cfg
# ntp authentication-key 2 sha1 7 123456
# ntp authentication-key 23 md5 7 123456
# ntp trusted-key 23
# ntp authenticate
# ntp local-interface Ethernet1
# ntp qos dscp 10
# ntp server vrf vrf01 10.1.1.1 prefer burst
# ntp server vrf vrf01 11.21.1.1 prefer burst minpoll 13
# ntp server vrf vrf01 25.1.1.1 maxpoll 15 key 2
# ntp serve ip access-group acl01 in
# ntp serve ipv6 access-group acl02 in

- name: parse configs
  arista.eos.eos_ntp_global:
    running_config: "{{ lookup('file', './parsed_ntp_global.cfg') }}"
    state: parsed
  tags:
    - parsed

# Module Execution
# "parsed": {
#         "authenticate": {
#             "enable": true
#         },
#         "authentication_keys": [
#             {
#                 "algorithm": "sha1",
#                 "encryption": 7,
#                 "id": 2,
#                 "key": "VALUE_SPECIFIED_IN_NO_LOG_PARAMETER"
#             },
#             {
#                 "algorithm": "md5",
#                 "encryption": 7,
#                 "id": 23,
#                 "key": "VALUE_SPECIFIED_IN_NO_LOG_PARAMETER"
#             }
#         ],
#         "local_interface": "Ethernet1",
#         "qos_dscp": 10,
#         "serve": {
#             "access_lists": [
#                 {
#                     "acls": [
#                         {
#                             "acl_name": "acl01",
#                             "direction": "in"
#                         }
#                     ],
#                     "afi": "ip"
#                 },
#                 {
#                     "acls": [
#                         {
#                             "acl_name": "acl02",
#                             "direction": "in"
#                         }
#                     ],
#                     "afi": "ipv6"
#                 }
#             ]
#         },
#         "servers": [
#             {
#                 "burst": true,
#                 "prefer": true,
#                 "server": "10.1.1.1",
#                 "vrf": "vrf01"
#             },
#             {
#                 "burst": true,
#                 "minpoll": 13,
#                 "prefer": true,
#                 "server": "11.21.1.1",
#                 "vrf": "vrf01"
#             },
#             {
#                 "key": 2,
#                 "maxpoll": 15,
#                 "server": "25.1.1.1",
#                 "vrf": "vrf01"
#             }
#         ],
#         "trusted_key": "23"
#     }
# }

# using Gathered
# Device config:
# localhost(config)#show running-config | section ntp
# ntp authentication-key 2 sha1 7 123456
# ntp authentication-key 23 md5 7 123456
# ntp trusted-key 23
# ntp authenticate
# ntp local-interface Ethernet1
# ntp qos dscp 10
# ntp server vrf vrf01 10.1.1.1 prefer burst
# ntp server vrf vrf01 25.1.1.1 maxpoll 15 key 2
# ntp serve ip access-group acl01 in
# ntp serve ipv6 access-group acl02 in
# localhost(config)#

- name: gather configs
  arista.eos.eos_ntp_global:
    state: gathered
  tags:
    - gathered

# Module Execution

#   "gathered": {
#         "authenticate": {
#             "enable": true
#         },
#         "authentication_keys": [
#             {
#                 "algorithm": "sha1",
#                 "encryption": 7,
#                 "id": 2,
#                 "key": "VALUE_SPECIFIED_IN_NO_LOG_PARAMETER"
#             },
#             {
#                 "algorithm": "md5",
#                 "encryption": 7,
#                 "id": 23,
#                 "key": "VALUE_SPECIFIED_IN_NO_LOG_PARAMETER"
#             }
#         ],
#         "local_interface": "Ethernet1",
#         "qos_dscp": 10,
#         "serve": {
#             "access_lists": [
#                 {
#                     "acls": [
#                         {
#                             "acl_name": "acl01",
#                             "direction": "in"
#                         }
#                     ],
#                     "afi": "ip"
#                 },
#                 {
#                     "acls": [
#                         {
#                             "acl_name": "acl02",
#                             "direction": "in"
#                         }
#                     ],
#                     "afi": "ipv6"
#                 }
#             ]
#         },
#         "servers": [
#             {
#                 "burst": true,
#                 "prefer": true,
#                 "server": "10.1.1.1",
#                 "vrf": "vrf01"
#             },
#             {
#                 "key_id": 2,
#                 "maxpoll": 15,
#                 "server": "25.1.1.1",
#                 "vrf": "vrf01"
#             }
#         ],
#         "trusted_key": "23"
#     },
#     "invocation": {
#         "module_args": {
#             "config": null,
#             "running_config": null,
#             "state": "gathered"
#         }
#     }
# }

# using rendered:

- name: Render provided configuration
  arista.eos.eos_ntp_global:
    config:
      authenticate:
        enable: true
      authentication_keys:
        - id: 2
          algorithm: "sha1"
          encryption: 7
          key: "123456"
        - id: 23
          algorithm: "md5"
          encryption: 7
          key: "123456"
      local_interface: "Ethernet1"
      qos_dscp: 10
      trusted_key: 23
      servers:
        - server: "10.1.1.1"
          vrf: "vrf01"
          burst: true
          prefer: true
        - server: "25.1.1.1"
          vrf: "vrf01"
          maxpoll: 15
          key_id: 2
      serve:
        access_lists:
          - afi: "ip"
            acls:
              - acl_name: "acl01"
                direction: "in"
          - afi: "ipv6"
            acls:
              - acl_name: "acl02"
                direction: "in"
    state: rendered

# Module Execution:
# "rendered": [
#         "ntp serve ip access-group acl01 in",
#         "ntp serve ipv6 access-group acl02 in",
#         "ntp authentication-key 2 sha1 7 ********",
#         "ntp authentication-key 23 md5 7 ********",
#         "ntp server vrf vrf01 10.1.1.1 burst prefer",
#         "ntp server vrf vrf01 25.1.1.1 key 2 maxpoll 15",
#         "ntp authenticate",
#         "ntp local-interface Ethernet1",
#         "ntp qos dscp 10",
#         "ntp trusted-key 23"
#     ]
#
"""

RETURN = """
before:
  description: The configuration prior to the module execution.
  returned: when I(state) is C(merged), C(replaced), C(overridden), C(deleted) or C(purged)
  type: dict
  sample: >
    This output will always be in the same format as the
    module argspec.
after:
  description: The resulting configuration after module execution.
  returned: when changed
  type: dict
  sample: >
    This output will always be in the same format as the
    module argspec.
commands:
  description: The set of commands pushed to the remote device.
  returned: when I(state) is C(merged), C(replaced), C(overridden), C(deleted) or C(purged)
  type: list
  sample:
    - ntp master stratum 2
    - ntp peer 198.51.100.1 use-vrf test maxpoll 7
    - ntp authentication-key 10 md5 wawyhanx2 7
    - ntp access-group peer PeerAcl1
    - ntp access-group peer PeerAcl2
    - ntp access-group query-only QueryAcl1
rendered:
  description: The provided configuration in the task rendered in device-native format (offline).
  returned: when I(state) is C(rendered)
  type: list
  sample:

    - ntp authentication-key 2 sha1 7 123456
    - ntp authentication-key 23 md5 7 123456
    - ntp trusted-key 23
    - ntp authenticate
    - ntp local-interface Ethernet1
    - ntp qos dscp 10
    - ntp server vrf vrf01 10.1.1.1 prefer burst
    - ntp server vrf vrf01 25.1.1.1 maxpoll 15 key 2
    - ntp serve ip access-group acl01 in
    - ntp serve ipv6 access-group acl02 in

gathered:
  description: Facts about the network resource gathered from the remote device as structured data.
  returned: when I(state) is C(gathered)
  type: list
  sample: >
    This output will always be in the same format as the
    module argspec.
parsed:
  description: The device native config provided in I(running_config) option parsed into structured data as per module argspec.
  returned: when I(state) is C(parsed)
  type: list
  sample: >
    This output will always be in the same format as the
    module argspec.
"""

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.arista.eos.plugins.module_utils.network.eos.argspec.ntp_global.ntp_global import (
    Ntp_globalArgs,
)
from ansible_collections.arista.eos.plugins.module_utils.network.eos.config.ntp_global.ntp_global import (
    Ntp_global,
)


def main():
    """
    Main entry point for module execution

    :returns: the result form module invocation
    """
    module = AnsibleModule(
        argument_spec=Ntp_globalArgs.argument_spec,
        mutually_exclusive=[["config", "running_config"]],
        required_if=[
            ["state", "merged", ["config"]],
            ["state", "replaced", ["config"]],
            ["state", "overridden", ["config"]],
            ["state", "rendered", ["config"]],
            ["state", "parsed", ["running_config"]],
        ],
        supports_check_mode=True,
    )

    result = Ntp_global(module).execute_module()
    module.exit_json(**result)


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
# Copyright 2023 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function


__metaclass__ = type

#############################################
#                WARNING                    #
#############################################
#
# This file is auto generated by the
# ansible.content_builder.
#
# Manually editing this file is not advised.
#
# To update the argspec make the desired changes
# in the documentation in the module file and re-run
# ansible.content_builder commenting out
# the path to external 'docstring' in build.yaml.
#
##############################################

"""
The arg spec for the ios_ospf_interfaces module
"""


class Ospf_interfacesArgs(object):  # pylint: disable=R0903
    """The arg spec for the ios_ospf_interfaces module"""

    argument_spec = {
        "config": {
            "type": "list",
            "elements": "dict",
            "options": {
                "name": {"type": "str", "required": True},
                "address_family": {
                    "type": "list",
                    "elements": "dict",
                    "options": {
                        "afi": {
                            "type": "str",
                            "choices": ["ipv4", "ipv6"],
                            "required": True,
                        },
                        "process": {
                            "type": "dict",
                            "options": {
                                "id": {"type": "int"},
                                "area_id": {"type": "str"},
                                "secondaries": {"type": "bool"},
                                "instance_id": {"type": "int"},
                            },
                        },
                        "adjacency": {"type": "bool"},
                        "authentication": {
                            "type": "dict",
                            "options": {
                                "key_chain": {"type": "str", "no_log": True},
                                "message_digest": {"type": "bool"},
                                "null": {"type": "bool"},
                            },
                        },
                        "bfd": {"type": "bool"},
                        "cost": {
                            "type": "dict",
                            "options": {
                                "interface_cost": {"type": "int"},
                                "dynamic_cost": {
                                    "type": "dict",
                                    "options": {
                                        "default": {"type": "int"},
                                        "hysteresis": {
                                            "type": "dict",
                                            "options": {
                                                "percent": {"type": "int"},
                                                "threshold": {"type": "int"},
                                            },
                                        },
                                        "weight": {
                                            "type": "dict",
                                            "options": {
                                                "l2_factor": {"type": "int"},
                                                "latency": {"type": "int"},
                                                "oc": {"type": "bool"},
                                                "resources": {"type": "int"},
                                                "throughput": {"type": "int"},
                                            },
                                        },
                                    },
                                },
                            },
                        },
                        "database_filter": {"type": "bool"},
                        "dead_interval": {
                            "type": "dict",
                            "options": {
                                "time": {"type": "int"},
                                "minimal": {"type": "int"},
                            },
                        },
                        "demand_circuit": {
                            "type": "dict",
                            "options": {
                                "enable": {"type": "bool"},
                                "ignore": {"type": "bool"},
                                "disable": {"type": "bool"},
                            },
                        },
                        "flood_reduction": {"type": "bool"},
                        "hello_interval": {"type": "int"},
                        "lls": {"type": "bool"},
                        "manet": {
                            "type": "dict",
                            "options": {
                                "cost": {
                                    "type": "dict",
                                    "options": {
                                        "percent": {"type": "int"},
                                        "threshold": {"type": "int"},
                                    },
                                },
                                "link_metrics": {
                                    "type": "dict",
                                    "options": {
                                        "set": {"type": "bool"},
                                        "cost_threshold": {"type": "int"},
                                    },
                                },
                            },
                        },
                        "mtu_ignore": {"type": "bool"},
                        "multi_area": {
                            "type": "dict",
                            "options": {
                                "id": {"type": "int"},
                                "cost": {"type": "int"},
                            },
                        },
                        "neighbor": {
                            "type": "dict",
                            "options": {
                                "address": {"type": "str"},
                                "cost": {"type": "int"},
                                "database_filter": {"type": "bool"},
                                "poll_interval": {"type": "int"},
                                "priority": {"type": "int"},
                            },
                        },
                        "network": {
                            "type": "dict",
                            "options": {
                                "broadcast": {"type": "bool"},
                                "manet": {"type": "bool"},
                                "non_broadcast": {"type": "bool"},
                                "point_to_multipoint": {"type": "bool"},
                                "point_to_point": {"type": "bool"},
                            },
                        },
                        "prefix_suppression": {"type": "bool"},
                        "priority": {"type": "int"},
                        "resync_timeout": {"type": "int"},
                        "retransmit_interval": {"type": "int"},
                        "shutdown": {"type": "bool"},
                        "transmit_delay": {"type": "int"},
                        "ttl_security": {
                            "type": "dict",
                            "options": {
                                "set": {"type": "bool"},
                                "hops": {"type": "int"},
                            },
                        },
                    },
                },
            },
        },
        "running_config": {"type": "str"},
        "state": {
            "type": "str",
            "choices": [
                "merged",
                "replaced",
                "overridden",
                "deleted",
                "gathered",
                "rendered",
                "parsed",
            ],
            "default": "merged",
        },
    }  # pylint: disable=C0301



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: snmpv2_write_community_credential_v1
short_description: Resource module for Snmpv2 Write Community Credential V1
description:
- Manage operations create and update of the resource Snmpv2 Write Community Credential V1.
- Adds global SNMP write community.
- Updates global SNMP write community.
version_added: '3.1.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  comments:
    description: Comments to identify the credential.
    type: str
  credentialType:
    description: Credential type to identify the application that uses the credential.
    type: str
  description:
    description: Name/Description of the credential.
    type: str
  instanceUuid:
    description: Credential UUID.
    type: str
  writeCommunity:
    description: SNMP write community. NO!$DATA!$ for no value change.
    type: str
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Discovery CreateSNMPWriteCommunityV1
  description: Complete reference of the CreateSNMPWriteCommunityV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!create-snmp-write-community
- name: Cisco DNA Center documentation for Discovery UpdateSNMPWriteCommunityV1
  description: Complete reference of the UpdateSNMPWriteCommunityV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!update-snmp-write-community
notes:
  - SDK Method used are
    discovery.Discovery.create_snmp_write_community_v1,
    discovery.Discovery.update_snmp_write_community_v1,

  - Paths used are
    post /dna/intent/api/v1/global-credential/snmpv2-write-community,
    put /dna/intent/api/v1/global-credential/snmpv2-write-community,

"""

EXAMPLES = r"""
- name: Create
  cisco.dnac.snmpv2_write_community_credential_v1:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: present
    comments: string
    credentialType: string
    description: string
    writeCommunity: string

- name: Update all
  cisco.dnac.snmpv2_write_community_credential_v1:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: present
    comments: string
    credentialType: string
    description: string
    instanceUuid: string
    writeCommunity: string

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": {
        "taskId": "string",
        "url": "string"
      },
      "version": "string"
    }
"""



from __future__ import absolute_import, division, print_function

__metaclass__ = type

import pytest

from awx.main.models import Instance
from django.test.utils import override_settings


@pytest.mark.django_db
def test_peers_adding_and_removing(run_module, admin_user):
    with override_settings(IS_K8S=True):
        result = run_module(
            'instance',
            {'hostname': 'hopnode', 'node_type': 'hop', 'node_state': 'installed', 'listener_port': 6789},
            admin_user,
        )
        assert result['changed']

        hop_node = Instance.objects.get(pk=result.get('id'))

        assert hop_node.node_type == 'hop'

        address = hop_node.receptor_addresses.get(pk=result.get('id'))
        assert address.port == 6789

        result = run_module(
            'instance',
            {'hostname': 'executionnode', 'node_type': 'execution', 'node_state': 'installed', 'peers': ['hopnode']},
            admin_user,
        )
        assert result['changed']

        execution_node = Instance.objects.get(pk=result.get('id'))

        assert set(execution_node.peers.all()) == {address}

        result = run_module(
            'instance',
            {'hostname': 'executionnode', 'node_type': 'execution', 'node_state': 'installed', 'peers': []},
            admin_user,
        )

        assert result['changed']
        assert set(execution_node.peers.all()) == set()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: sda_authentication_profiles_v1
short_description: Resource module for Sda Authentication Profiles V1
description:
- Manage operation update of the resource Sda Authentication Profiles V1.
- Updates an authentication profile based on user input.
version_added: '6.14.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  payload:
    description: Sda Authentication Profiles's payload.
    elements: dict
    suboptions:
      authenticationOrder:
        description: First authentication method.
        type: str
      authenticationProfileName:
        description: The default host authentication template (updating this field is
          not allowed).
        type: str
      dot1xToMabFallbackTimeout:
        description: 802.1x Timeout.
        type: int
      fabricId:
        description: ID of the fabric this authentication profile is assigned to (updating
          this field is not allowed).
        type: str
      id:
        description: ID of the authentication profile (updating this field is not allowed).
        type: str
      isBpduGuardEnabled:
        description: Enable/disable BPDU Guard. Only applicable when authenticationProfileName
          is set to "Closed Authentication" (defaults to true).
        type: bool
      numberOfHosts:
        description: Number of Hosts.
        type: str
      wakeOnLan:
        description: Wake on LAN.
        type: bool
    type: list
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for SDA UpdateAuthenticationProfileV1
  description: Complete reference of the UpdateAuthenticationProfileV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!update-authentication-profile
notes:
  - SDK Method used are
    sda.Sda.update_authentication_profile_v1,

  - Paths used are
    put /dna/intent/api/v1/sda/authenticationProfiles,

"""

EXAMPLES = r"""
- name: Update all
  cisco.dnac.sda_authentication_profiles_v1:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: present
    payload:
    - authenticationOrder: string
      authenticationProfileName: string
      dot1xToMabFallbackTimeout: 0
      fabricId: string
      id: string
      isBpduGuardEnabled: true
      numberOfHosts: string
      wakeOnLan: true

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": {
        "taskId": "string",
        "url": "string"
      },
      "version": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright 2019 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

#############################################
#                WARNING                    #
#############################################
#
# This file is auto generated by the resource
#   module builder playbook.
#
# Do not edit this file manually.
#
# Changes to this file will be over written
#   by the resource module builder.
#
# Changes should be made in the model used to
#   generate this file or in the resource module
#   builder template.
#
#############################################

"""
The module file for eos_lldp_global
"""

from __future__ import absolute_import, division, print_function


__metaclass__ = type


DOCUMENTATION = """
module: eos_lldp_global
short_description: LLDP resource module
description:
- This module manages Global Link Layer Discovery Protocol (LLDP) settings on Arista
  EOS devices.
version_added: 1.0.0
author: Nathaniel Case (@Qalthos)
notes:
- Tested against Arista EOS 4.24.6F
- This module works with connection C(network_cli). See the L(EOS Platform Options,../network/user_guide/platform_eos.html).
options:
  config:
    description: The provided global LLDP configuration.
    type: dict
    suboptions:
      holdtime:
        description:
        - Specifies the holdtime (in sec) to be sent in packets.
        type: int
      reinit:
        description:
        - Specifies the delay (in sec) for LLDP initialization on any interface.
        type: int
      timer:
        description:
        - Specifies the rate at which LLDP packets are sent (in sec).
        type: int
      tlv_select:
        description:
        - Specifies the LLDP TLVs to enable or disable.
        type: dict
        suboptions:
          link_aggregation:
            description:
            - Enable or disable link aggregation TLV.
            type: bool
          management_address:
            description:
            - Enable or disable management address TLV.
            type: bool
          max_frame_size:
            description:
            - Enable or disable maximum frame size TLV.
            type: bool
          port_description:
            description:
            - Enable or disable port description TLV.
            type: bool
          system_capabilities:
            description:
            - Enable or disable system capabilities TLV.
            type: bool
          system_description:
            description:
            - Enable or disable system description TLV.
            type: bool
          system_name:
            description:
            - Enable or disable system name TLV.
            type: bool
  running_config:
    description:
    - This option is used only with state I(parsed).
    - The value of this option should be the output received from the EOS device by
      executing the command B(show running-config | section lldp).
    - The state I(parsed) reads the configuration from C(running_config) option and
      transforms it into Ansible structured data as per the resource module's argspec
      and the value is then returned in the I(parsed) key within the result.
    type: str
  state:
    description:
    - The state of the configuration after module completion.
    type: str
    choices:
    - merged
    - replaced
    - deleted
    - rendered
    - gathered
    - parsed
    default: merged

"""
EXAMPLES = """
# Using merged
#
# ------------
# Before State
# ------------
#
# veos# show run | section lldp
# lldp timer 3000
# lldp holdtime 100
# lldp reinit 5
# no lldp tlv-select management-address
# no lldp tlv-select system-description

- name: Merge provided LLDP configuration with the existing configuration
  arista.eos.eos_lldp_global:
    config:
      holdtime: 100
      tlv_select:
        management_address: false
        port_description: false
        system_description: true
    state: merged

# -----------
# After state
# -----------
#
# veos# show run | section lldp
# lldp timer 3000
# lldp holdtime 100
# lldp reinit 5
# no lldp tlv-select management-address
# no lldp tlv-select port-description


# Using replaced
#
# ------------
# Before State
# ------------
#
# veos# show run | section lldp
# lldp timer 3000
# lldp holdtime 100
# lldp reinit 5
# no lldp tlv-select management-address
# no lldp tlv-select system-description

- name: Replace existing LLDP device configuration with provided configuration
  arista.eos.eos_lldp_global:
    config:
      holdtime: 100
      tlv_select:
        management_address: false
        port_description: false
        system_description: true
    state: replaced

# -----------
# After state
# -----------
#
# veos# show run | section lldp
# lldp holdtime 100
# no lldp tlv-select management-address
# no lldp tlv-select port-description


# Using deleted
#
# ------------
# Before State
# ------------
#
# veos# show run | section lldp
# lldp timer 3000
# lldp holdtime 100
# lldp reinit 5
# no lldp tlv-select management-address
# no lldp tlv-select system-description

- name: Delete existing LLDP configurations from the device
  arista.eos.eos_lldp_global:
    state: deleted

# -----------
# After state
# -----------
#
# veos# show run | section ^lldp

# Using rendered:

- name: Use Rendered to convert the structured data to native config
  arista.eos.eos_lldp_global:
    config:
      holdtime: 100
      tlv_select:
        management_address: false
        port_description: false
        system_description: true
    state: rendered

# -----------
# Output
# -----------
#
# rendered:
#   - "lldp holdtime 100"
#   - "no lldp tlv-select management-address"
#   - "no lldp tlv-select port-description"

# Using parsed

# parsed.cfg

# lldp timer 3000
# lldp holdtime 100
# lldp reinit 5
# no lldp tlv-select management-address
# no lldp tlv-select system-description

- name: Use parsed to convert native configs to structured data
  arista.eos.lldp_global:
    running_config: "{{ lookup('file', 'parsed.cfg') }}"
    state: parsed

# -----------
# Output
# -----------

#    parsed:
#      holdtime: 100
#      timer 3000
#      reinit 5
#      tlv_select:
#        management_address: False
#        port_description: False
#        system_description: True

# Using gathered:
# native config:
# lldp timer 3000
# lldp holdtime 100
# lldp reinit 5
# no lldp tlv-select management-address
# no lldp tlv-select system-description


- name: Gather lldp_global facts from the device
  arista.eos.lldp_global:
    state: gathered

# -----------
# Output
# -----------

#    gathered:
#      holdtime: 100
#      timer 3000
#      reinit 5
#      tlv_select:
#        management_address: False
#        port_description: False
#        system_description: True
"""
RETURN = """
before:
  description: The configuration as structured data prior to module invocation.
  returned: always
  type: dict
  sample: >
    The configuration returned will always be in the same format
     of the parameters above.
after:
  description: The configuration as structured data after module completion.
  returned: when changed
  type: dict
  sample: >
    The configuration returned will always be in the same format
     of the parameters above.
commands:
  description: The set of commands pushed to the remote device.
  returned: always
  type: list
  sample: ['lldp holdtime 100', 'no lldp timer', 'lldp tlv-select system-description']
"""


from ansible.module_utils.basic import AnsibleModule

from ansible_collections.arista.eos.plugins.module_utils.network.eos.argspec.lldp_global.lldp_global import (
    Lldp_globalArgs,
)
from ansible_collections.arista.eos.plugins.module_utils.network.eos.config.lldp_global.lldp_global import (
    Lldp_global,
)


def main():
    """
    Main entry point for module execution

    :returns: the result form module invocation
    """

    required_if = [
        ("state", "merged", ("config",)),
        ("state", "replaced", ("config",)),
        ("state", "overridden", ("config",)),
        ("state", "rendered", ("config",)),
        ("state", "parsed", ("running_config",)),
    ]
    mutually_exclusive = [("config", "running_config")]
    module = AnsibleModule(
        argument_spec=Lldp_globalArgs.argument_spec,
        required_if=required_if,
        supports_check_mode=True,
        mutually_exclusive=mutually_exclusive,
    )

    result = Lldp_global(module).execute_module()
    module.exit_json(**result)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)


DOCUMENTATION = r"""
---
module: configuration_template_project
short_description: Resource module for Configuration Template Project
description:
- This module represents an alias of the module configuration_template_project_v1
version_added: '3.1.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  createTime:
    description: Create time of project.
    type: int
  description:
    description: Description of project.
    type: str
  id:
    description: UUID of project.
    type: str
  lastUpdateTime:
    description: Update time of project.
    type: int
  name:
    description: Name of project.
    type: str
  projectId:
    description: ProjectId path parameter. ProjectId(UUID) of project to be deleted.
    type: str
  tags:
    description: Configuration Template Project's tags.
    elements: dict
    suboptions:
      id:
        description: UUID of tag.
        type: str
      name:
        description: Name of tag.
        type: str
    type: list
  templates:
    description: List of templates within the project.
    elements: dict
    type: list
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Configuration Templates CreateProjectV1
  description: Complete reference of the CreateProjectV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!create-project
- name: Cisco DNA Center documentation for Configuration Templates DeletesTheProjectV1
  description: Complete reference of the DeletesTheProjectV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!deletes-the-project
- name: Cisco DNA Center documentation for Configuration Templates UpdateProjectV1
  description: Complete reference of the UpdateProjectV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!update-project
notes:
  - SDK Method used are
    configuration_templates.ConfigurationTemplates.create_project_v1,
    configuration_templates.ConfigurationTemplates.deletes_the_project_v1,
    configuration_templates.ConfigurationTemplates.update_project_v1,

  - Paths used are
    post /dna/intent/api/v1/template-programmer/project,
    delete /dna/intent/api/v1/template-programmer/project/{projectId},
    put /dna/intent/api/v1/template-programmer/project,
  - It should be noted that this module is an alias of configuration_template_project_v1

"""

EXAMPLES = r"""
- name: Create
  cisco.dnac.configuration_template_project:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: present
    createTime: 0
    description: string
    id: string
    lastUpdateTime: 0
    name: string
    tags:
    - id: string
      name: string
    templates:
    - {}

- name: Update all
  cisco.dnac.configuration_template_project:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: present
    createTime: 0
    description: string
    id: string
    lastUpdateTime: 0
    name: string
    tags:
    - id: string
      name: string
    templates: {}

- name: Delete by id
  cisco.dnac.configuration_template_project:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: absent
    projectId: string

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": {
        "taskId": "string",
        "url": "string"
      },
      "version": "string"
    }
"""



# -*- coding: utf-8 -*-

# Copyright: (c) 2021, Felix Fontein <felix@fontein.de>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

"""Provide version object to compare version numbers."""

# This should be directly imported by modules, rather than importing from here.
# The import is being kept for backwards compatibility.
from ansible.module_utils.compat.version import LooseVersion  # pylint: disable=unused-import



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2017 Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: ec2_vpc_vpn
version_added: 1.0.0
version_added_collection: community.aws
short_description: Create, modify, and delete EC2 VPN connections
description:
  - This module creates, modifies, and deletes VPN connections.
  - Idempotence is achieved by using the O(filters) option or specifying the VPN connection identifier.
author:
  - Sloane Hertel (@s-hertel)
options:
  state:
    description:
      - The desired state of the VPN connection.
    choices: ["present", "absent"]
    default: present
    required: false
    type: str
  customer_gateway_id:
    description:
      - The ID of the customer gateway.
    type: str
  connection_type:
    description:
      - The type of VPN connection.
      - At this time only V(ipsec.1) is supported.
    default: "ipsec.1"
    type: str
  vpn_gateway_id:
    description:
      - The ID of the virtual private gateway.
      - Mutually exclusive with O(transit_gateway_id).
    type: str
  vpn_connection_id:
    description:
      - The ID of the VPN connection. Required to modify or delete a connection if the filters option does not provide a unique match.
    type: str
  static_only:
    description:
      - Indicates whether the VPN connection uses static routes only. Static routes must be used for devices that don't support BGP.
    default: false
    type: bool
    required: false
  transit_gateway_id:
    description:
      - The ID of the transit gateway.
      - Mutually exclusive with O(vpn_gateway_id).
    type: str
    version_added: 6.2.0
  local_ipv4_network_cidr:
    description:
      - The IPv4 CIDR on the customer gateway (on-premises) side of the VPN connection.
    required: false
    type: str
    default: "0.0.0.0/0"
    version_added: 9.0.0
  tunnel_options:
    description:
      - An optional list object containing no more than two dict members, each of which may contain O(tunnel_options.TunnelInsideCidr)
        and/or O(tunnel_options.PreSharedKey) keys with appropriate string values.
        AWS defaults will apply in absence of either of the aforementioned keys.
    required: false
    type: list
    elements: dict
    default: []
    suboptions:
        TunnelInsideCidr:
            type: str
            description:
              - The range of inside IPv4 addresses for the tunnel.
        TunnelInsideIpv6Cidr:
            type: str
            description:
              - The range of inside IPv6 addresses for the tunnel.
            version_added: 9.0.0
        PreSharedKey:
            type: str
            description:
              - The pre-shared key (PSK) to establish initial authentication between the virtual private gateway and customer gateway.
  filters:
    description:
      - An alternative to using O(vpn_connection_id). If multiple matches are found, O(vpn_connection_id) is required.
        If one of the following suboptions is a list of items to filter by, only one item needs to match to find the VPN
        that correlates. e.g. if the filter O(filters.cidr) is V(["194.168.2.0/24", "192.168.2.0/24"]) and the VPN route only has the
        destination cidr block of V(192.168.2.0/24) it will be found with this filter (assuming there are not multiple
        VPNs that are matched). Another example, if the filter O(filters.vpn) is equal to V(["vpn-ccf7e7ad", "vpn-cb0ae2a2"]) and one
        of of the VPNs has the state deleted (exists but is unmodifiable) and the other exists and is not deleted,
        it will be found via this filter.
    suboptions:
      cgw-config:
        description:
          - The customer gateway configuration of the VPN as a string (in the format of the return value) or a list of those strings.
      static-routes-only:
        description:
          - The type of routing; V(true) or V(false).
        type: bool
      cidr:
        description:
          - The destination cidr of the VPN's route as a string or a list of those strings.
      bgp:
        description:
          - The BGP ASN number associated with a BGP device. Only works if the connection is attached.
            This filtering option is currently not working.
      vpn:
        description:
          - The VPN connection id as a string or a list of those strings.
      vgw:
        description:
          - The virtual private gateway as a string or a list of those strings.
      tag-keys:
        description:
          - The key of a tag as a string or a list of those strings.
      tag-values:
        description:
          - The value of a tag as a string or a list of those strings.
      tags:
        description:
          - A dict of key value pairs.
        type: dict
      cgw:
        description:
          - The customer gateway id as a string or a list of those strings.
    type: dict
    default: {}
  routes:
    description:
      - Routes to add to the connection.
    type: list
    elements: str
    default: []
  purge_routes:
    description:
      - Whether or not to delete VPN connections routes that are not specified in the task.
    type: bool
    default: false
  wait_timeout:
    description:
      - How long, in seconds, before wait gives up.
    default: 600
    type: int
    required: false
  delay:
    description:
      - The time, in seconds, to wait before checking operation again.
    required: false
    type: int
    default: 15
extends_documentation_fragment:
  - amazon.aws.region.modules
  - amazon.aws.common.modules
  - amazon.aws.tags
  - amazon.aws.boto3
"""

EXAMPLES = r"""
# Note: These examples do not set authentication details, see the AWS Guide for details.

- name: Create a VPN connection with vpn_gateway_id
  amazon.aws.ec2_vpc_vpn:
    state: "present"
    vpn_gateway_id: "vgw-XXXXXXXX"
    customer_gateway_id: "cgw-XXXXXXXX"

- name: Attach a vpn connection to transit gateway
  amazon.aws.ec2_vpc_vpn:
    state: "present"
    transit_gateway_id: "tgw-XXXXXXXX"
    customer_gateway_id: "cgw-XXXXXXXX"

- name: Modify VPN connection tags
  amazon.aws.ec2_vpc_vpn:
    state: "present"
    vpn_connection_id: "vpn-XXXXXXXX"
    tags:
      Name: "ansible-tag-1"
      Other: "ansible-tag-2"

- name: Delete a connection
  amazon.aws.ec2_vpc_vpn:
    vpn_connection_id: "vpn-XXXXXXXX"
    state: "absent"

- name: Modify VPN tags (identifying VPN by filters)
  amazon.aws.ec2_vpc_vpn:
    state: "present"
    filters:
      cidr: "194.168.1.0/24"
      tag-keys:
        - "Ansible"
        - "Other"
    tags:
      New: "Tag"
    purge_tags: true
    static_only: true

- name: Set up VPN with tunnel options utilizing 'TunnelInsideCidr' only
  amazon.aws.ec2_vpc_vpn:
    state: "present"
    filters:
      vpn: "vpn-XXXXXXXX"
    static_only: true
    tunnel_options:
      - TunnelInsideCidr: "169.254.100.1/30"
      - TunnelInsideCidr: "169.254.100.5/30"

- name: Add routes and remove any preexisting ones
  amazon.aws.ec2_vpc_vpn:
    state: "present"
    filters:
      vpn: "vpn-XXXXXXXX"
    routes:
      - "195.168.2.0/24"
      - "196.168.2.0/24"
    purge_routes: true

- name: Remove all routes
  amazon.aws.ec2_vpc_vpn:
    state: "present"
    vpn_connection_id: "vpn-XXXXXXXX"
    routes: []
    purge_routes: true

- name: Delete a VPN identified by filters
  amazon.aws.ec2_vpc_vpn:
    state: "absent"
    filters:
      tags:
        Ansible: "Tag"
"""

RETURN = r"""
changed:
  description: If the VPN connection has changed.
  type: bool
  returned: always
  sample: true
customer_gateway_configuration:
  description: The configuration of the VPN connection.
  returned: O(state=present)
  type: str
customer_gateway_id:
  description: The customer gateway connected via the connection.
  type: str
  returned: O(state=present)
  sample: "cgw-1220c87b"
gateway_association_state:
  description: The current state of the gateway association.
  type: str
  returned: O(state=present)
  sample: "associated"
vpn_gateway_id:
  description: The virtual private gateway connected via the connection.
  type: str
  returned: O(state=present)
  sample: "vgw-cb0ae2a2"
transit_gateway_id:
  description: The transit gateway id to which the vpn connection can be attached.
  type: str
  returned: O(state=present)
  sample: "tgw-cb0ae2a2"
options:
  description: The VPN connection options.
  type: list
  elements: dict
  returned: O(state=present)
  contains:
    static_routes_only:
      description: If the VPN connection only allows static routes.
      returned: O(state=present)
      type: bool
      sample: true
    enable_acceleration:
      description: Indicates whether acceleration is enabled for the VPN connection.
      returned: O(state=present)
      type: bool
      sample: false
    local_ipv4_network_cidr:
      description: The IPv4 CIDR on the customer gateway (on-premises) side of the VPN connection.
      returned: O(state=present)
      type: str
      sample: "0.0.0.0/0"
    outside_ip_address_type:
      description: The external IP address of the VPN tunnel.
      returned: O(state=present)
      type: str
      sample: "PublicIpv4"
    remote_ipv4_network_cidr:
      description: The IPv4 CIDR on the Amazon Web Services side of the VPN connection.
      returned: O(state=present)
      type: str
      sample: "0.0.0.0/0"
    tunnel_inside_ip_version:
      description: Indicates whether the VPN tunnels process IPv4 or IPv6 traffic.
      returned: O(state=present)
      type: str
      sample: "ipv4"
    tunnel_options:
      description: Indicates the VPN tunnel options.
      returned: O(state=present)
      type: list
      elements: dict
      sample: [{
                "log_options": {
                    "cloud_watch_log_options": {
                        "log_enabled": false
                    }
                },
                "outside_ip_address": "34.225.101.10",
                "pre_shared_key": "8n7hnjNE8zhIt4VpMOIfcrw6XnUTHLW9",
                "tunnel_inside_cidr": "169.254.31.8/30"
            }]
      contains:
        log_options:
          description: Options for logging VPN tunnel activity.
          returned: O(state=present)
          type: dict
          contains:
            cloud_watch_log_options:
              description: Options for sending VPN tunnel logs to CloudWatch.
              type: dict
              returned: O(state=present)
        outside_ip_address:
          description: The external IP address of the VPN tunnel.
          type: str
          returned: O(state=present)
        pre_shared_key:
          description:
            - The pre-shared key (PSK) to establish initial authentication between the
              virtual private gateway and the customer gateway.
          type: str
          returned: O(state=present)
        tunnel_inside_cidr:
          description: The range of inside IPv4 addresses for the tunnel.
          type: str
          returned: O(state=present)
routes:
  description: The routes of the VPN connection.
  type: list
  returned: O(state=present)
  sample: [{
              "destination_cidr_block": "192.168.1.0/24",
              "state": "available"
            }]
  contains:
    destination_cidr_block:
      description:
        - The CIDR block associated with the local subnet of the customer data center.
      type: str
      returned: O(state=present)
    source:
      description: Indicates how the routes were provided.
      type: str
      returned: O(state=present)
    state:
      description: The current state of the static route.
      type: str
      returned: O(state=present)
state:
  description: The status of the VPN connection.
  type: str
  returned: O(state=present)
  sample: "available"
tags:
  description: The tags associated with the connection.
  type: dict
  returned: O(state=present)
  sample: {
      "name": "ansible-test",
      "other": "tag"
    }
type:
  description: The type of VPN connection (currently only ipsec.1 is available).
  type: str
  returned: O(state=present)
  sample: "ipsec.1"
vgw_telemetry:
  type: list
  returned: O(state=present)
  description: The telemetry for the VPN tunnel.
  sample: [{
            "accepted_route_count": 0,
            "last_status_change": "2024-09-30T13:12:33+00:00",
            "outside_ip_address": "34.225.101.10",
            "status": "DOWN",
            "status_message": "IPSEC IS DOWN"
        }]
  contains:
    accepted_route_count:
      type: int
      returned: O(state=present)
      description: The number of accepted routes.
    last_status_change:
      type: str
      returned: O(state=present)
      description: The date and time of the last change in status.
    outside_ip_address:
      type: str
      returned: O(state=present)
      description:
        - The Internet-routable IP address of the virtual private gateway's outside interface.
    status:
      type: str
      returned: O(state=present)
      description: The status of the VPN tunnel.
    status_message:
      type: str
      returned: O(state=present)
      description: If an error occurs, a description of the error.
    certificate_arn:
      description: The Amazon Resource Name of the virtual private gateway tunnel endpoint certificate.
      returned: when a private certificate is used for authentication
      type: str
      sample: "arn:aws:acm:us-east-1:123456789012:certificate/c544d8ce-20b8-4fff-98b0-example"
vpn_connection_id:
  description: The identifier for the VPN connection.
  type: str
  returned: O(state=present)
  sample: "vpn-781e0e19"
"""

try:
    from botocore.exceptions import WaiterError
except ImportError:
    pass  # Handled by AnsibleAWSModule

from typing import Any
from typing import Dict
from typing import List
from typing import NoReturn
from typing import Optional
from typing import Tuple
from typing import Union

from ansible.module_utils._text import to_text
from ansible.module_utils.common.dict_transformations import camel_dict_to_snake_dict

from ansible_collections.amazon.aws.plugins.module_utils.ec2 import AnsibleEC2Error
from ansible_collections.amazon.aws.plugins.module_utils.ec2 import create_vpn_connection
from ansible_collections.amazon.aws.plugins.module_utils.ec2 import create_vpn_connection_route
from ansible_collections.amazon.aws.plugins.module_utils.ec2 import delete_vpn_connection
from ansible_collections.amazon.aws.plugins.module_utils.ec2 import delete_vpn_connection_route
from ansible_collections.amazon.aws.plugins.module_utils.ec2 import describe_vpn_connections
from ansible_collections.amazon.aws.plugins.module_utils.ec2 import ensure_ec2_tags
from ansible_collections.amazon.aws.plugins.module_utils.modules import AnsibleAWSModule
from ansible_collections.amazon.aws.plugins.module_utils.tagging import boto3_tag_list_to_ansible_dict
from ansible_collections.amazon.aws.plugins.module_utils.tagging import compare_aws_tags


def find_vpn_connection(
    client, module: AnsibleAWSModule, vpn_connection_id: Optional[str] = None
) -> Union[None, Dict[str, Any]]:
    """Looks for a unique VPN connection. Uses find_connection_response() to return the connection found, None,
    or raise an error if there were multiple viable connections."""

    filters = module.params.get("filters")
    params: Dict[str, Any] = {}

    # vpn_connection_id may be provided via module option; takes precedence over any filter values
    if not vpn_connection_id and module.params.get("vpn_connection_id"):
        vpn_connection_id = module.params["vpn_connection_id"]

    if not isinstance(vpn_connection_id, list) and vpn_connection_id:
        vpn_connection_id = [to_text(vpn_connection_id)]
    elif isinstance(vpn_connection_id, list):
        vpn_connection_id = [to_text(connection) for connection in vpn_connection_id]

    formatted_filter: List = []
    # if vpn_connection_id is provided it will take precedence over any filters since it is a unique identifier
    if not vpn_connection_id:
        formatted_filter = create_filter(module, filters)

    if vpn_connection_id:
        params["VpnConnectionIds"] = vpn_connection_id
    params["Filters"] = formatted_filter

    # see if there is a unique matching connection
    try:
        existing_conn = describe_vpn_connections(client, **params)
    except AnsibleEC2Error as e:
        module.fail_json_aws(e, msg="Failed while describing VPN connection.")

    return find_connection_response(module, connections=existing_conn)


def add_routes(client, module: AnsibleAWSModule, vpn_connection_id: str, routes_to_add: List[Dict[str, Any]]) -> bool:
    changed: bool = False
    for route in routes_to_add:
        try:
            changed |= create_vpn_connection_route(client, vpn_connection_id, route)
        except AnsibleEC2Error as e:
            module.fail_json_aws(e, msg=f"Failed while adding route {route} to the VPN connection {vpn_connection_id}.")
    return changed


def remove_routes(
    client, module: AnsibleAWSModule, vpn_connection_id: str, routes_to_remove: List[Dict[str, Any]]
) -> bool:
    changed: bool = False
    for route in routes_to_remove:
        try:
            changed |= delete_vpn_connection_route(client, vpn_connection_id, route)
        except AnsibleEC2Error as e:
            module.fail_json_aws(e, msg=f"Failed to remove route {route} from the VPN connection {vpn_connection_id}.")
    return changed


def create_filter(module, provided_filters: Dict[str, Any]) -> List[Dict[str, Any]]:
    """Creates a filter using the user-specified parameters and unmodifiable options that may have been specified in the task"""

    boto3ify_filter = {
        "cgw-config": "customer-gateway-configuration",
        "static-routes-only": "option.static-routes-only",
        "cidr": "route.destination-cidr-block",
        "bgp": "bgp-asn",
        "vpn": "vpn-connection-id",
        "vgw": "vpn-gateway-id",
        "tag-keys": "tag-key",
        "tag-values": "tag-value",
        "tags": "tag",
        "cgw": "customer-gateway-id",
    }

    # unmodifiable options and their filter name counterpart
    param_to_filter = {
        "customer_gateway_id": "customer-gateway-id",
        "vpn_gateway_id": "vpn-gateway-id",
        "transit_gateway_id": "transit-gateway-id",
        "vpn_connection_id": "vpn-connection-id",
    }

    flat_filter_dict = {}
    formatted_filter: List = []

    for raw_param in dict(provided_filters):
        # fix filter names to be recognized by boto3
        if raw_param in boto3ify_filter:
            param = boto3ify_filter[raw_param]
            provided_filters[param] = provided_filters.pop(raw_param)
        elif raw_param in list(boto3ify_filter.items()):
            param = raw_param
        else:
            module.fail_json(msg=f"{raw_param} is not a valid filter.")

        # reformat filters with special formats
        if param == "tag":
            for key in provided_filters[param]:
                formatted_key = "tag:" + key
                if isinstance(provided_filters[param][key], list):
                    flat_filter_dict[formatted_key] = str(provided_filters[param][key])
                else:
                    flat_filter_dict[formatted_key] = [str(provided_filters[param][key])]
        elif param == "option.static-routes-only":
            flat_filter_dict[param] = [str(provided_filters[param]).lower()]
        else:
            if isinstance(provided_filters[param], list):
                flat_filter_dict[param] = provided_filters[param]
            else:
                flat_filter_dict[param] = [str(provided_filters[param])]

    # if customer_gateway, vpn_gateway, or vpn_connection was specified in the task but not the filter, add it
    for param in param_to_filter:
        if param_to_filter[param] not in flat_filter_dict and module.params.get(param):
            flat_filter_dict[param_to_filter[param]] = [module.params.get(param)]

    # change the flat dict into something boto3 will understand
    formatted_filter = [{"Name": key, "Values": value} for key, value in flat_filter_dict.items()]

    return formatted_filter


def find_connection_response(module, connections: Optional[List[Dict[str, Any]]] = None) -> Optional[Dict[str, Any]]:
    """Determine if there is a viable unique match in the connections described. Returns the unique VPN connection if one is found,
    returns None if the connection does not exist, raise an error if multiple matches are found."""

    # Found no connections
    if not connections:
        return None

    # Too many results
    elif connections and len(connections) > 1:
        viable = []
        for each in connections:
            # deleted connections are not modifiable
            if each["State"] not in ("deleted", "deleting"):
                viable.append(each)
        if len(viable) == 1:
            # Found one viable result; return unique match
            return viable[0]
        elif len(viable) == 0:
            # Found a result but it was deleted already; since there was only one viable result create a new one
            return None
        else:
            module.fail_json(
                msg=(
                    "More than one matching VPN connection was found. "
                    "To modify or delete a VPN please specify vpn_connection_id or add filters."
                )
            )

    # Found unique match
    elif connections and len(connections) == 1:
        # deleted connections are not modifiable
        if connections[0]["State"] not in ("deleted", "deleting"):
            return connections[0]


def create_connection(
    client,
    module: AnsibleAWSModule,
    customer_gateway_id: Optional[str],
    static_only: Optional[bool],
    vpn_gateway_id: str,
    transit_gateway_id: str,
    connection_type: Optional[str],
    max_attempts: Optional[int],
    delay: Optional[int],
    local_ipv4_network_cidr: Optional[str],
    tunnel_options: Optional[List[Dict[str, Any]]] = None,
) -> Dict[str, Any]:
    """Creates a VPN connection"""

    options = {"StaticRoutesOnly": static_only, "LocalIpv4NetworkCidr": local_ipv4_network_cidr}

    if tunnel_options and len(tunnel_options) <= 2:
        t_opt = []
        for m in tunnel_options:
            # See Boto3 docs regarding 'create_vpn_connection'
            # tunnel options for allowed 'TunnelOptions' keys.
            if not isinstance(m, dict):
                raise TypeError("non-dict list member")
            t_opt.append(m)
        if t_opt:
            options["TunnelOptions"] = t_opt

    if not (customer_gateway_id and (vpn_gateway_id or transit_gateway_id)):
        module.fail_json(
            msg=(
                "No matching connection was found. To create a new connection you must provide "
                "customer_gateway_id and one of either transit_gateway_id or vpn_gateway_id."
            )
        )
    vpn_connection_params: Dict[str, Any] = {
        "Type": connection_type,
        "CustomerGatewayId": customer_gateway_id,
        "Options": options,
    }

    if vpn_gateway_id:
        vpn_connection_params["VpnGatewayId"] = vpn_gateway_id
    if transit_gateway_id:
        vpn_connection_params["TransitGatewayId"] = transit_gateway_id

    try:
        vpn = create_vpn_connection(client, **vpn_connection_params)
        client.get_waiter("vpn_connection_available").wait(
            VpnConnectionIds=[vpn["VpnConnectionId"]],
            WaiterConfig={"Delay": delay, "MaxAttempts": max_attempts},
        )
    except WaiterError as e:
        module.fail_json_aws(e, msg=f"Failed to wait for VPN connection {vpn['VpnConnectionId']} to be available")
    except AnsibleEC2Error as e:
        module.fail_json_aws(e, msg="Failed to create VPN connection")

    return vpn


def delete_connection(client, module: AnsibleAWSModule, vpn_connection_id: str) -> NoReturn:
    """Deletes a VPN connection"""

    delay = module.params.get("delay")
    max_attempts = module.params.get("wait_timeout") // delay

    try:
        delete_vpn_connection(client, vpn_connection_id)
        client.get_waiter("vpn_connection_deleted").wait(
            VpnConnectionIds=[vpn_connection_id], WaiterConfig={"Delay": delay, "MaxAttempts": max_attempts}
        )
    except WaiterError as e:
        module.fail_json_aws(e, msg=f"Failed to wait for VPN connection {vpn_connection_id} to be removed")
    except AnsibleEC2Error as e:
        module.fail_json_aws(e, msg=f"Failed to delete the VPN connection: {vpn_connection_id}")


def check_for_routes_update(client, module: AnsibleAWSModule, vpn_connection_id: str) -> Dict[str, Any]:
    """Determines if there are any routes that need to be updated. Ensures non-modifiable attributes aren't expected to change."""
    routes = module.params.get("routes")
    purge_routes = module.params.get("purge_routes")

    vpn_connection = find_vpn_connection(client, module, vpn_connection_id)
    current_attrs = camel_dict_to_snake_dict(vpn_connection)

    # Initialize changes dict
    changes: Dict[str, Any] = {"routes_to_add": [], "routes_to_remove": []}

    # Get changes to routes
    if "Routes" in vpn_connection:
        current_routes = [route["DestinationCidrBlock"] for route in vpn_connection["Routes"]]
        if purge_routes:
            changes["routes_to_remove"] = [old_route for old_route in current_routes if old_route not in routes]
        changes["routes_to_add"] = [new_route for new_route in routes if new_route not in current_routes]

    # Check if nonmodifiable attributes are attempted to be modified
    for attribute in current_attrs:
        if attribute in ("tags", "routes", "state"):
            continue
        elif attribute == "options":
            will_be = module.params.get("static_only")
            is_now = bool(current_attrs[attribute]["static_routes_only"])
            attribute = "static_only"
        elif attribute == "type":
            will_be = module.params.get("connection_type")
            is_now = current_attrs[attribute]
        else:
            is_now = current_attrs[attribute]
            will_be = module.params.get(attribute)

        if will_be is not None and to_text(will_be) != to_text(is_now):
            module.fail_json(
                msg=(
                    f"You cannot modify {attribute}, the current value of which is {is_now}. Modifiable VPN connection"
                    f" attributes are tags and routes. The value you tried to change it to is {will_be}."
                )
            )

    return changes


def make_changes(client, module: AnsibleAWSModule, vpn_connection_id: str, changes: Dict[str, Any]) -> bool:
    """changes is a dict with the keys 'routes_to_add', 'routes_to_remove',
    the values of which are lists (generated by check_for_routes_update()).
    """
    changed: bool = False

    if module.params.get("tags") is not None:
        changed |= ensure_ec2_tags(
            client,
            module,
            vpn_connection_id,
            resource_type="vpn-connection",
            tags=module.params.get("tags"),
            purge_tags=module.params.get("purge_tags"),
        )

    if changes["routes_to_add"]:
        changed |= add_routes(client, module, vpn_connection_id, changes["routes_to_add"])

    if changes["routes_to_remove"]:
        changed |= remove_routes(client, module, vpn_connection_id, changes["routes_to_remove"])

    return changed


def get_check_mode_results(
    module_params: Dict[str, Any], vpn_connection_id: Optional[str] = None, current_state: Optional[str] = None
) -> Tuple[bool, Dict[str, Any]]:
    """Returns the changes that would be made to a VPN Connection"""
    changed: bool = False
    results: Dict[str, Any] = {
        "customer_gateway_configuration": "",
        "customer_gateway_id": module_params.get("customer_gateway_id"),
        "vpn_gateway_id": module_params.get("vpn_gateway_id"),
        "transit_gateway_id": module_params.get("transit_gateway_id"),
        "options": {"static_routes_only": module_params.get("static_only")},
        "routes": [module_params.get("routes")],
    }

    present_tags = module_params.get("tags")
    # get combined current tags and tags to set
    if present_tags is None:
        pass
    elif current_state and "Tags" in current_state:
        current_tags = boto3_tag_list_to_ansible_dict(current_state["Tags"])
        tags_to_add, tags_to_remove = compare_aws_tags(current_tags, present_tags, module_params.get("purge_tags"))
        changed |= bool(tags_to_remove) or bool(tags_to_add)
        if module_params.get("purge_tags"):
            current_tags = {}
        current_tags.update(present_tags)
        results["tags"] = current_tags
    elif module_params.get("tags"):
        changed = True

    if present_tags:
        results["tags"] = present_tags

    # get combined current routes and routes to add
    present_routes = module_params.get("routes")
    if current_state and "Routes" in current_state:
        current_routes = [route["DestinationCidrBlock"] for route in current_state["Routes"]]
        if module_params.get("purge_routes"):
            if set(current_routes) != set(present_routes):
                changed = True
        elif set(present_routes) != set(current_routes):
            if not set(present_routes) < set(current_routes):
                changed = True
            present_routes.extend([route for route in current_routes if route not in present_routes])
    elif module_params.get("routes"):
        changed = True
    results["routes"] = [{"destination_cidr_block": cidr, "state": "available"} for cidr in present_routes]

    # return the vpn_connection_id if it's known
    if vpn_connection_id:
        results["vpn_connection_id"] = vpn_connection_id
    else:
        changed = True
        results["vpn_connection_id"] = "vpn-XXXXXXXX"

    return changed, results


def ensure_present(
    client, module: AnsibleAWSModule, vpn_connection: Optional[Dict[str, Any]]
) -> Tuple[bool, Dict[str, Any]]:
    """Creates and adds tags to a VPN connection. If the connection already exists update tags."""
    changed: bool = False
    delay = module.params.get("delay")
    max_attempts = module.params.get("wait_timeout") // delay

    # No match but vpn_connection_id was specified.
    if not vpn_connection and module.params.get("vpn_connection_id"):
        module.fail_json(msg="There is no VPN connection available or pending with that id. Did you delete it?")

    # Unique match was found. Check if attributes provided differ.
    elif vpn_connection:
        vpn_connection_id = vpn_connection["VpnConnectionId"]
        # check_for_update returns a dict with the keys routes_to_add, routes_to_remove
        changes = check_for_routes_update(client, module, vpn_connection_id)

        if module.check_mode:
            return get_check_mode_results(module.params, vpn_connection_id, current_state=vpn_connection)

        changed |= make_changes(client, module, vpn_connection_id, changes)

    # No match was found. Create and tag a connection and add routes.
    else:
        changed = True

        if module.check_mode:
            return get_check_mode_results(module.params)

        vpn_connection = create_connection(
            client,
            module,
            customer_gateway_id=module.params.get("customer_gateway_id"),
            static_only=module.params.get("static_only"),
            vpn_gateway_id=module.params.get("vpn_gateway_id"),
            transit_gateway_id=module.params.get("transit_gateway_id"),
            connection_type=module.params.get("connection_type"),
            local_ipv4_network_cidr=module.params.get("local_ipv4_network_cidr"),
            tunnel_options=module.params.get("tunnel_options"),
            max_attempts=max_attempts,
            delay=delay,
        )

        changes = check_for_routes_update(client, module, vpn_connection["VpnConnectionId"])
        make_changes(client, module, vpn_connection["VpnConnectionId"], changes)

    # get latest version if a change has been made and make tags output nice before returning it
    if vpn_connection:
        vpn_connection = find_vpn_connection(client, module, vpn_connection["VpnConnectionId"])
        if "Tags" in vpn_connection:
            vpn_connection["Tags"] = boto3_tag_list_to_ansible_dict(vpn_connection["Tags"])

    return (changed, vpn_connection)


def ensure_absent(client, module: AnsibleAWSModule, vpn_connection: Dict[str, Any]) -> bool:
    """Deletes a VPN connection if it exists."""
    changed: bool = False

    if vpn_connection:
        changed = True

        if module.check_mode:
            return changed

        delete_connection(client, module, vpn_connection["VpnConnectionId"])

    return changed


def main():
    argument_spec = dict(
        state=dict(type="str", default="present", choices=["present", "absent"]),
        filters=dict(type="dict", default={}),
        vpn_gateway_id=dict(type="str"),
        tags=dict(type="dict", aliases=["resource_tags"]),
        connection_type=dict(default="ipsec.1", type="str"),
        transit_gateway_id=dict(type="str"),
        local_ipv4_network_cidr=dict(type="str", default="0.0.0.0/0"),
        tunnel_options=dict(
            no_log=True,
            type="list",
            default=[],
            elements="dict",
            options=dict(
                TunnelInsideCidr=dict(type="str"),
                TunnelInsideIpv6Cidr=dict(type="str"),
                PreSharedKey=dict(type="str", no_log=True),
            ),
        ),
        static_only=dict(default=False, type="bool"),
        customer_gateway_id=dict(type="str"),
        vpn_connection_id=dict(type="str"),
        purge_tags=dict(type="bool", default=True),
        routes=dict(type="list", default=[], elements="str"),
        purge_routes=dict(type="bool", default=False),
        wait_timeout=dict(type="int", default=600),
        delay=dict(type="int", default=15),
    )
    mutually_exclusive = [
        ["vpn_gateway_id", "transit_gateway_id"],
    ]

    module = AnsibleAWSModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        mutually_exclusive=mutually_exclusive,
    )
    client = module.client("ec2")

    response: Dict[str, Any] = {}
    state = module.params.get("state")

    vpn_connection = find_vpn_connection(client, module)

    if state == "present":
        changed, response = ensure_present(client, module, vpn_connection)
    elif state == "absent":
        changed = ensure_absent(client, module, vpn_connection)

    module.exit_json(changed=changed, **camel_dict_to_snake_dict(response))


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)


DOCUMENTATION = r"""
---
module: reports_view_group_info
short_description: Information module for Reports View Group Info
description:
- This module represents an alias of the module reports_view_group_v1_info
version_added: '3.1.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
  viewGroupId:
    description:
    - ViewGroupId path parameter. ViewGroupId of viewgroup.
    type: str
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Reports GetAllViewGroupsV1
  description: Complete reference of the GetAllViewGroupsV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!get-all-view-groups
- name: Cisco DNA Center documentation for Reports GetViewsForAGivenViewGroupV1
  description: Complete reference of the GetViewsForAGivenViewGroupV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!get-views-for-a-given-view-group
notes:
  - SDK Method used are
    reports.Reports.get_all_view_groups_v1,
    reports.Reports.get_views_for_a_given_view_group_v1,

  - Paths used are
    get /dna/intent/api/v1/data/view-groups,
    get /dna/intent/api/v1/data/view-groups/{viewGroupId},
  - It should be noted that this module is an alias of reports_view_group_v1_info

"""

EXAMPLES = r"""
- name: Get all Reports View Group Info
  cisco.dnac.reports_view_group_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
  register: result

- name: Get Reports View Group Info by id
  cisco.dnac.reports_view_group_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    viewGroupId: string
  register: result

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "viewGroupId": "string",
      "views": [
        {
          "description": "string",
          "viewId": "string",
          "viewName": "string"
        }
      ]
    }
"""



#!/usr/bin/python
#
# Copyright (c) 2024 xuzhang3 (@xuzhang3), Fred-sun (@Fred-sun)
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: azure_rm_akscredentials_info

version_added: "2.3.0"

short_description: List Azure Kubernetes Service Credentials facts

description:
    - List Azure Kubernetes Service Credentials facts.

options:
    cluster_name:
        description:
            - Azure Kubernetes Service or all Azure Kubernetes Services.
        type: str
        required: true
    resource_group:
        description:
            - The resource group to search for the desired Azure Kubernetes Service.
        type: str
        required: true
    show_admin_credentials:
        description:
            - Whether list Cluster Admin Credentials.
        type: bool
        default: false
    show_user_credentials:
        description:
            - Whether list Cluster User Credentials.
        type: bool
        default: false
    show_monitor_credentials:
        description:
            - Whether list Cluster Monitoring User Credentials.
        type: bool
        default: false

extends_documentation_fragment:
    - azure.azcollection.azure

author:
    - xuzhang3 (@xuzhang3)
    - Fred Sun (@Fred-sun)
'''

EXAMPLES = '''
- name: Get managecluster admin credentials
  azure_rm_akscredentials_info:
    resource_group: "{{ resource_group }}"
    cluster_name: cluter_name
    show_admin_credentials: true

- name: Get managecluster user credentials
  azure_rm_akscredentials_info:
    resource_group: "{{ resource_group }}"
    cluster_name: cluster_name
    show_user_credentials: true

- name: Get managecluster monitor user credentials
  azure_rm_akscredentials_info:
    resource_group: "{{ resource_group }}"
    cluster_name: cluster_name
    show_monitor_credentials: true
'''

RETURN = '''
cluster_credentials:
    description:
        - Lists the cluster user, admin or monitoring user credentials of a managed cluster
    returned: always
    type: complex
    contains:
        cluster_name:
            description:
                - Azure Kubernetes Service or all Azure Kubernetes Services.
            type: str
            returned: always
            sample: testcluster01
        resource_group:
            description:
                - The resource group to search for the desired Azure Kubernetes Service.
            type: str
            returned: always
            sample:
        name:
            description:
                - The name of the credential.
            type: str
            returned: always
            sample:
        value:
            description:
                -  Base64-encoded Kubernetes configuration file.
            type: str
            returned: always
            sample: "apiVersion: ************************"
'''

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase

try:
    from azure.core.exceptions import HttpResponseError
except Exception:
    # handled in azure_rm_common
    pass

AZURE_OBJECT_CLASS = 'managedClustersCredentials'


class AzureRMAksCredentialsInfo(AzureRMModuleBase):
    """Utility class to get Azure Kubernetes Service Credentials facts"""

    def __init__(self):

        self.module_args = dict(
            cluster_name=dict(type='str', required=True),
            resource_group=dict(type='str', required=True),
            show_admin_credentials=dict(type='bool', default=False),
            show_user_credentials=dict(type='bool', default=False),
            show_monitor_credentials=dict(type='bool', default=False),
        )

        self.results = dict(
            changed=False,
            cluster_credentials=[],
        )

        mutually_exclusive = [('show_admin_credentials', 'show_user_credentials', 'show_monitor_credentials')]

        super(AzureRMAksCredentialsInfo, self).__init__(
            derived_arg_spec=self.module_args,
            supports_check_mode=True,
            supports_tags=False,
            mutually_exclusive=mutually_exclusive,
            facts_module=True
        )

    def exec_module(self, **kwargs):

        for key in self.module_args:
            setattr(self, key, kwargs[key])

        if self.show_user_credentials:
            self.results['cluster_credentials'] = self.get_user_credentials()
        elif self.show_admin_credentials:
            self.results['cluster_credentials'] = self.get_admin_credentials()
        elif self.show_monitor_credentials:
            self.results['cluster_credentials'] = self.get_monitor_credentials()

        self.results['resource_group'] = self.resource_group
        self.results['cluster_name'] = self.cluster_name
        return self.results

    def get_user_credentials(self):
        """Get The Azure Kubernetes Service USER Credentials"""
        response = None

        try:
            response = self.managedcluster_client.managed_clusters.list_cluster_user_credentials(self.resource_group, self.cluster_name)
        except HttpResponseError as ec:
            self.fail("Lists the cluster user credentials of a managed cluster Failed, Exception as {0}".format(ec))
        return [self.format_response(item) for item in response.kubeconfigs]

    def get_admin_credentials(self):
        """Get The Azure Kubernetes Service Admin Credentials"""
        response = None

        try:
            response = self.managedcluster_client.managed_clusters.list_cluster_admin_credentials(self.resource_group, self.cluster_name)
        except HttpResponseError as ec:
            self.fail("Lists the cluster admin credentials of a managed cluster Failed, Exception as {0}".format(ec))

        return [self.format_response(item) for item in response.kubeconfigs]

    def get_monitor_credentials(self):
        """Get The Azure Kubernetes Service Monitor Credentials"""
        response = None

        try:
            response = self.managedcluster_client.managed_clusters.list_cluster_monitoring_user_credentials(self.resource_group, self.cluster_name)
        except HttpResponseError as ec:
            self.fail("Lists the cluster monitoring credentials of a managed cluster Failed, Exception as {0}".format(ec))
        return [self.format_response(item) for item in response.kubeconfigs]

    def format_response(self, item):
        if not item:
            return
        return dict(name=item.name, value=item.value.decode('utf-8'))


def main():
    """Main module execution code path"""

    AzureRMAksCredentialsInfo()


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)


DOCUMENTATION = r"""
---
module: tag
short_description: Resource module for Tag
description:
- This module represents an alias of the module tag_v1
version_added: '3.1.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  description:
    description: Description of the tag.
    type: str
  dynamicRules:
    description: Tag's dynamicRules.
    elements: dict
    suboptions:
      memberType:
        description: MemberType of the tag (e.g. Networkdevice, interface).
        type: str
      rules:
        description: Tag's rules.
        suboptions:
          items:
            description: Items details,multiple rules can be defined by items(e.g. "items"
              {"operation" "ILIKE", "name" "managementIpAddress", "value" "%10%"}, {"operation"
              "ILIKE", "name" "hostname", "value" "%NA%"} ).
            elements: str
            type: list
          name:
            description: Name of the parameter (e.g. For interface portName,adminStatus,speed,status,description.
              For networkdevice family,series,hostname,managementIpAddress,groupNameHierarchy,softwareVersion).
            type: str
          operation:
            description: Opeartion used in the rules (e.g. OR,IN,EQ,LIKE,ILIKE,AND).
            type: str
          value:
            description: Value of the parameter (e.g. For portName 1/0/1,for adminStatus,status
              up/down, for speed any integer value, for description any valid string,
              for family switches, for series C3650, for managementIpAddress 10.197.124.90,
              groupNameHierarchy Global, softwareVersion 16.9.1).
            type: str
          values:
            description: Values of the parameter,Only one of the value or values can
              be used for the given parameter. (for managementIpAddress e.g. "10.197.124.90","10.197.124.91").
            elements: str
            type: list
        type: dict
    type: list
  id:
    description: Mandatory instanceUuid of the tag that needs to be updated.
    type: str
  instanceTenantId:
    description: InstanceTenantId generated for the tag.
    type: str
  name:
    description: Name of the tag.
    type: str
  systemTag:
    description: True for system created tags, false for user defined tags.
    type: bool
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Tag CreateTagV1
  description: Complete reference of the CreateTagV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!create-tag
- name: Cisco DNA Center documentation for Tag DeleteTagV1
  description: Complete reference of the DeleteTagV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!delete-tag
- name: Cisco DNA Center documentation for Tag UpdateTagV1
  description: Complete reference of the UpdateTagV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!update-tag
notes:
  - SDK Method used are
    tag.Tag.create_tag_v1,
    tag.Tag.delete_tag_v1,
    tag.Tag.update_tag_v1,

  - Paths used are
    post /dna/intent/api/v1/tag,
    delete /dna/intent/api/v1/tag/{id},
    put /dna/intent/api/v1/tag,
  - It should be noted that this module is an alias of tag_v1

"""

EXAMPLES = r"""
- name: Update all
  cisco.dnac.tag:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: present
    description: string
    dynamicRules:
    - memberType: string
      rules:
        items:
        - string
        name: string
        operation: string
        value: string
        values:
        - string
    id: string
    instanceTenantId: string
    name: string
    systemTag: true

- name: Create
  cisco.dnac.tag:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: present
    description: string
    dynamicRules:
    - memberType: string
      rules:
        items:
        - string
        name: string
        operation: string
        value: string
        values:
        - string
    id: string
    instanceTenantId: string
    name: string
    systemTag: true

- name: Delete by id
  cisco.dnac.tag:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: absent
    id: string

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "version": "string",
      "response": {
        "taskId": "string",
        "url": "string"
      }
    }
"""



from __future__ import absolute_import, division, print_function

__metaclass__ = type

import pytest

from awx.main.models import Organization, Inventory, InventorySource, Project


@pytest.fixture
def base_inventory():
    org = Organization.objects.create(name='test-org')
    inv = Inventory.objects.create(name='test-inv', organization=org)
    return inv


@pytest.fixture
def project(base_inventory):
    return Project.objects.create(
        name='test-proj',
        organization=base_inventory.organization,
        scm_type='git',
        scm_url='https://github.com/ansible/test-playbooks.git',
    )


@pytest.mark.django_db
def test_inventory_source_create(run_module, admin_user, base_inventory, project):
    source_path = '/var/lib/awx/example_source_path/'
    result = run_module(
        'inventory_source',
        dict(name='foo', inventory=base_inventory.name, state='present', source='scm', source_path=source_path, source_project=project.name),
        admin_user,
    )
    assert result.pop('changed', None), result

    inv_src = InventorySource.objects.get(name='foo')
    assert inv_src.inventory == base_inventory
    result.pop('invocation')
    assert result == {
        'id': inv_src.id,
        'name': 'foo',
    }


@pytest.mark.django_db
def test_create_inventory_source_implied_org(run_module, admin_user):
    org = Organization.objects.create(name='test-org')
    inv = Inventory.objects.create(name='test-inv', organization=org)

    # Credential is not required for ec2 source, because of IAM roles
    result = run_module('inventory_source', dict(name='Test Inventory Source', inventory='test-inv', source='ec2', state='present'), admin_user)
    assert result.pop('changed', None), result

    inv_src = InventorySource.objects.get(name='Test Inventory Source')
    assert inv_src.inventory == inv

    result.pop('invocation')
    assert result == {
        "name": "Test Inventory Source",
        "id": inv_src.id,
    }


@pytest.mark.django_db
def test_create_inventory_source_multiple_orgs(run_module, admin_user):
    org = Organization.objects.create(name='test-org')
    Inventory.objects.create(name='test-inv', organization=org)

    # make another inventory by same name in another org
    org2 = Organization.objects.create(name='test-org-number-two')
    inv2 = Inventory.objects.create(name='test-inv', organization=org2)

    result = run_module(
        'inventory_source',
        dict(name='Test Inventory Source', inventory=inv2.name, organization='test-org-number-two', source='ec2', state='present'),
        admin_user,
    )
    assert result.pop('changed', None), result

    inv_src = InventorySource.objects.get(name='Test Inventory Source')
    assert inv_src.inventory == inv2

    result.pop('invocation')
    assert result == {
        "name": "Test Inventory Source",
        "id": inv_src.id,
    }


@pytest.mark.django_db
def test_falsy_value(run_module, admin_user, base_inventory):
    result = run_module('inventory_source', dict(name='falsy-test', inventory=base_inventory.name, source='ec2', update_on_launch=True), admin_user)
    assert not result.get('failed', False), result.get('msg', result)
    assert result.get('changed', None), result

    inv_src = InventorySource.objects.get(name='falsy-test')
    assert inv_src.update_on_launch is True

    result = run_module('inventory_source', dict(name='falsy-test', inventory=base_inventory.name, source='ec2', update_on_launch=False), admin_user)

    inv_src.refresh_from_db()
    assert inv_src.update_on_launch is False


# Tests related to source-specific parameters
#
# We want to let the API return issues with "this doesn't support that", etc.
#
# GUI OPTIONS:
# - - - - - - - manual:	file:	scm:	ec2:	gce	azure_rm	vmware	sat	openstack	rhv	tower	custom
# credential		?	?	o	o	r	r		r	r		r		r	r	o
# source_project	?	?	r	-	-	-		-	-		-		-	-	-
# source_path		?	?	r	-	-	-		-	-		-		-	-	-
# scm_branch		?	?	r	-	-	-		-	-		-		-	-	-
# verbosity			?	?	o	o	o	o		o	o		o		o	o	o
# overwrite			?	?	o	o	o	o		o	o		o		o	o	o
# overwrite_vars	?	?	o	o	o	o		o	o		o		o	o	o
# update_on_launch	?	?	o	o	o	o		o	o		o		o	o	o
# UoPL          	?	?	o	-	-	-		-	-		-		-	-	-
# source_vars*		?	?	-	o	-	o		o	o		o		-	-	-
# environmet vars*	?	?	o	-	-	-		-	-		-		-	-	o
# source_script		?	?	-	-	-	-		-	-		-		-	-	r
#
# UoPL - update_on_project_launch
# * - source_vars are labeled environment_vars on project and custom sources


@pytest.mark.django_db
def test_missing_required_credential(run_module, admin_user, base_inventory):
    result = run_module('inventory_source', dict(name='Test Azure Source', inventory=base_inventory.name, source='azure_rm', state='present'), admin_user)
    assert result.pop('failed', None) is True, result

    assert 'Credential is required for a cloud source' in result.get('msg', '')


@pytest.mark.django_db
def test_source_project_not_for_cloud(run_module, admin_user, base_inventory, project):
    result = run_module(
        'inventory_source',
        dict(name='Test ec2 Inventory Source', inventory=base_inventory.name, source='ec2', state='present', source_project=project.name),
        admin_user,
    )
    assert result.pop('failed', None) is True, result

    assert 'Cannot set source_project if not SCM type' in result.get('msg', '')


@pytest.mark.django_db
def test_source_path_not_for_cloud(run_module, admin_user, base_inventory):
    result = run_module(
        'inventory_source',
        dict(name='Test ec2 Inventory Source', inventory=base_inventory.name, source='ec2', state='present', source_path='where/am/I'),
        admin_user,
    )
    assert result.pop('failed', None) is True, result

    assert 'Cannot set source_path if not SCM type' in result.get('msg', '')


@pytest.mark.django_db
def test_scm_source_needs_project(run_module, admin_user, base_inventory):
    result = run_module(
        'inventory_source',
        dict(
            name='SCM inventory without project', inventory=base_inventory.name, state='present', source='scm', source_path='/var/lib/awx/example_source_path/'
        ),
        admin_user,
    )
    assert result.pop('failed', None), result

    assert 'Project required for scm type sources' in result.get('msg', '')
